@@ -1,6 +1,6 @@
 /*
     GNOME Commander - A GNOME based file manager 
-    Copyright (C) 2001-2002 Marcus Bjurman
+    Copyright (C) 2001-2003 Marcus Bjurman
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,7 +17,15 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */ 
 
+#ifndef __CAP_H__
+#define __CAP_H__
 
-void cap_cut_files (GList *files, GnomeCmdDir *dir);
-void cap_copy_files (GList *files, GnomeCmdDir *dir);
+#include "gnome-cmd-file-list.h"
+#include "gnome-cmd-dir.h"
+
+void cap_cut_files (GnomeCmdFileList *fl, GList *files);
+void cap_copy_files (GnomeCmdFileList *fl, GList *files);
 void cap_paste_files (GnomeCmdDir *dir);
+
+
+#endif // __CAP_H__
