@@ -21,7 +21,7 @@
 #define __GNOME_CMD_ADVRENAME_LEXER_H__
 
 
-#include "gnome-cmd-file-types.h"
+#include "gnome-cmd-file.h"
 
 void gnome_cmd_advrename_reset_counter(unsigned start, unsigned precision, unsigned step);
 void gnome_cmd_advrename_parse_fname(const char *fname);
