@@ -20,8 +20,6 @@
 #define __GNOME_CMD_CHMOD_DIALOG_H__
 
 
-BEGIN_GNOME_DECLS
-
 #define GNOME_CMD_CHMOD_DIALOG(obj) \
 	GTK_CHECK_CAST (obj, gnome_cmd_chmod_dialog_get_type (), GnomeCmdChmodDialog)
 #define GNOME_CMD_CHMOD_DIALOG_CLASS(klass) \
@@ -38,14 +36,14 @@ typedef struct _GnomeCmdChmodDialogClass GnomeCmdChmodDialogClass;
 
 struct _GnomeCmdChmodDialog
 {
-	GnomeDialog parent;
+	GnomeCmdDialog parent;
 	GnomeCmdChmodDialogPrivate *priv;
 };
 
 
 struct _GnomeCmdChmodDialogClass
 {
-	GnomeDialogClass parent_class;
+	GnomeCmdDialogClass parent_class;
 };
 
 
