@@ -23,10 +23,7 @@
 #include "gnome-cmd-includes.h"
 #include "gnome-cmd-chown-dialog.h"
 #include "gnome-cmd-chown-component.h"
-#include "gnome-cmd-file-funcs.h"
-#include "gnome-cmd-dir-funcs.h"
-#include "widget-factory.h"
-#include "dir_pool.h"
+#include "gnome-cmd-dir.h"
 #include "owner.h"
 
 
@@ -37,7 +34,7 @@ struct _GnomeCmdChownDialogPrivate
 	GtkWidget *recurse_check;
 };
 
-static GnomeDialogClass *parent_class = NULL;
+static GnomeCmdDialogClass *parent_class = NULL;
 
 
 static void do_chown (GnomeCmdFile *in_finfo, uid_t uid, gid_t gid, gboolean recurse)
@@ -50,20 +47,19 @@ static void do_chown (GnomeCmdFile *in_finfo, uid_t uid, gid_t gid, gboolean rec
 	ret = gnome_cmd_file_chown (in_finfo, uid, gid);
 
 	if (ret != GNOME_VFS_OK) {
+		gchar *fpath = gnome_cmd_file_get_real_path (in_finfo);
 		gchar *msg = g_strdup_printf (
-			_("Could not chown %s\n%s"),
-			gnome_cmd_file_get_path (in_finfo),
-			gnome_vfs_result_to_string (ret));
-		gnome_error_dialog (msg);
+			_("Could not chown %s\n%s"), fpath, gnome_vfs_result_to_string (ret));
+		create_error_dialog (msg);
 		g_free (msg);
+		g_free (fpath);
 	}
 	else if (!recurse) {
 		return;
 	}
 	
 	if (in_finfo->info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
-		const gchar *dir_uri = gnome_cmd_file_get_uri_str (in_finfo);
-		GnomeCmdDir *dir = dir_pool_get (dir_uri);
+		GnomeCmdDir *dir = GNOME_CMD_DIR (in_finfo);
 		GList *files, *tmp;
 
 		gnome_cmd_dir_ref (dir);
@@ -120,14 +116,14 @@ static void on_ok (GtkButton *button, GnomeCmdChownDialog *dialog)
 	}
 
 	gnome_cmd_file_list_free (dialog->priv->files);
-	gtk_widget_hide (GTK_WIDGET (dialog));
+	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
 
 static void on_cancel (GtkButton *button, GnomeCmdChownDialog *dialog)
 {
 	gnome_cmd_file_list_free (dialog->priv->files);
-	gtk_widget_hide (GTK_WIDGET (dialog));
+	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
 
@@ -166,7 +162,7 @@ class_init (GnomeCmdChownDialogClass *class)
 	object_class = GTK_OBJECT_CLASS (class);
 	widget_class = GTK_WIDGET_CLASS (class);
 
-	parent_class = gtk_type_class (gnome_dialog_get_type ());
+	parent_class = gtk_type_class (gnome_cmd_dialog_get_type ());
 	object_class->destroy = destroy;
 	widget_class->map = map;
 }
@@ -175,19 +171,18 @@ class_init (GnomeCmdChownDialogClass *class)
 static void
 init (GnomeCmdChownDialog *dialog)
 {
+	GtkWidget *chown_dialog = GTK_WIDGET (dialog);
 	GtkWidget *vbox;
-	GtkWidget *button;
 	
-	dialog->priv = g_new (GnomeCmdChownDialogPrivate, 1);
+	dialog->priv = g_new0 (GnomeCmdChownDialogPrivate, 1);
 
 	gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, FALSE, FALSE);
 	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
 	gtk_window_set_title (GTK_WINDOW (dialog), _("Chown"));
 
-	vbox = GNOME_DIALOG (dialog)->vbox;
-	gtk_object_set_data (GTK_OBJECT (dialog), "vbox", vbox);
-	gtk_widget_show (vbox);
-
+	vbox = create_vbox (chown_dialog, FALSE, 12);
+	gnome_cmd_dialog_add_expanding_category (GNOME_CMD_DIALOG (dialog), vbox);
+	
 	dialog->priv->chown_component = gnome_cmd_chown_component_new ();
 	gtk_widget_ref (dialog->priv->chown_component);
 	gtk_object_set_data_full (GTK_OBJECT (dialog),
@@ -200,26 +195,11 @@ init (GnomeCmdChownDialog *dialog)
 		GTK_WIDGET (dialog), _("Apply Recursively"), "check");
 	gtk_box_pack_start (GTK_BOX (vbox), dialog->priv->recurse_check, FALSE, FALSE, 0);
 
-	
-	gnome_dialog_append_button (GNOME_DIALOG (dialog), GNOME_STOCK_BUTTON_OK);
-	button = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog)->buttons)->data);
-	gtk_widget_ref (button);
-	gtk_object_set_data_full (GTK_OBJECT (dialog), "button", button,
-							  (GtkDestroyNotify) gtk_widget_unref);
-	gtk_widget_show (button);
-	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",
-						GTK_SIGNAL_FUNC (on_ok), dialog);
 
-	gnome_dialog_append_button (GNOME_DIALOG (dialog), GNOME_STOCK_BUTTON_CANCEL);
-	button = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog)->buttons)->data);
-	gtk_widget_ref (button);
-	gtk_object_set_data_full (GTK_OBJECT (dialog), "button", button,
-							  (GtkDestroyNotify) gtk_widget_unref);
-	gtk_widget_show (button);
-	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",
-						GTK_SIGNAL_FUNC (on_cancel), dialog);
+	gnome_cmd_dialog_add_button (GNOME_CMD_DIALOG (dialog), GNOME_STOCK_BUTTON_CANCEL,
+								 GTK_SIGNAL_FUNC (on_cancel), dialog);
+	gnome_cmd_dialog_add_button (GNOME_CMD_DIALOG (dialog), GNOME_STOCK_BUTTON_OK,
+								 GTK_SIGNAL_FUNC (on_ok), dialog);
 }
 
 
@@ -269,10 +249,7 @@ gnome_cmd_chown_dialog_get_type         (void)
 			(GtkClassInitFunc) NULL
 		};
 
-		dlg_type = gtk_type_unique (gnome_dialog_get_type (), &dlg_info);
+		dlg_type = gtk_type_unique (gnome_cmd_dialog_get_type (), &dlg_info);
 	}
 	return dlg_type;
 }
-
-
-
