@@ -20,8 +20,6 @@
 #define __GNOME_CMD_CHOWN_DIALOG_H__
 
 
-BEGIN_GNOME_DECLS
-
 #define GNOME_CMD_CHOWN_DIALOG(obj) \
 	GTK_CHECK_CAST (obj, gnome_cmd_chown_dialog_get_type (), GnomeCmdChownDialog)
 #define GNOME_CMD_CHOWN_DIALOG_CLASS(klass) \
@@ -38,14 +36,14 @@ typedef struct _GnomeCmdChownDialogClass GnomeCmdChownDialogClass;
 
 struct _GnomeCmdChownDialog
 {
-	GnomeDialog parent;
+	GnomeCmdDialog parent;
 	GnomeCmdChownDialogPrivate *priv;
 };
 
 
 struct _GnomeCmdChownDialogClass
 {
-	GnomeDialogClass parent_class;
+	GnomeCmdDialogClass parent_class;
 };
 
 
