@@ -291,7 +291,7 @@ draw_row (GtkCList     *clist,
 		if (gdk_rectangle_intersect (area, &cell_rectangle,
 									 &intersect_rectangle))
 			gdk_draw_rectangle (clist->clist_window,
-								widget->style->base_gc[GTK_STATE_ACTIVE],
+								widget->style->base_gc[GTK_STATE_NORMAL],
 								TRUE,
 								intersect_rectangle.x,
 								intersect_rectangle.y,
@@ -306,7 +306,7 @@ draw_row (GtkCList     *clist,
 			if (gdk_rectangle_intersect (area, &cell_rectangle,
 										 &intersect_rectangle))
 				gdk_draw_rectangle (clist->clist_window,
-									widget->style->base_gc[GTK_STATE_ACTIVE],
+									widget->style->base_gc[GTK_STATE_NORMAL],
 									TRUE,
 									intersect_rectangle.x,
 									intersect_rectangle.y,
@@ -322,7 +322,7 @@ draw_row (GtkCList     *clist,
     {
 		rect = &clip_rectangle;
 		gdk_draw_rectangle (clist->clist_window,
-							widget->style->base_gc[GTK_STATE_ACTIVE],
+							widget->style->base_gc[GTK_STATE_NORMAL],
 							TRUE,
 							cell_rectangle.x,
 							cell_rectangle.y,
@@ -335,7 +335,7 @@ draw_row (GtkCList     *clist,
 			cell_rectangle.y += clist->row_height + CELL_SPACING;
 
 			gdk_draw_rectangle (clist->clist_window,
-								widget->style->base_gc[GTK_STATE_ACTIVE],
+								widget->style->base_gc[GTK_STATE_NORMAL],
 								TRUE,
 								cell_rectangle.x,
 								cell_rectangle.y,
@@ -354,12 +354,13 @@ draw_row (GtkCList     *clist,
 		GtkStyle *style;
 		GdkGC *fg_gc;
 		GdkGC *bg_gc;
+		PangoLayout *layout;
+		PangoRectangle logical_rect;
 
 		gint width;
 		gint height;
 		gint pixmap_width;
 		gint offset = 0;
-		gint row_center_offset;
 
 		if (!clist->column[i].visible)
 			continue;
@@ -385,31 +386,33 @@ draw_row (GtkCList     *clist,
 		clip_rectangle.width -= (2 * COLUMN_INSET + CELL_SPACING +
 								 (i == last_column) * CELL_SPACING);
 
+
 		/* calculate real width for column justification */
+      
+		layout = _gtk_clist_create_cell_layout (clist, clist_row, i);
+		if (layout)
+		{
+			pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
+			width = logical_rect.width;
+		}
+		else
+			width = 0;
+
 		pixmap_width = 0;
 		offset = 0;
 		switch (clist_row->cell[i].type)
 		{
-			case GTK_CELL_TEXT:
-				width = gdk_string_width (style->font,
-										  GTK_CELL_TEXT (clist_row->cell[i])->text);
-				break;
 			case GTK_CELL_PIXMAP:
 				gdk_window_get_size (GTK_CELL_PIXMAP (clist_row->cell[i])->pixmap,
 									 &pixmap_width, &height);
-				width = pixmap_width;
+				width += pixmap_width;
 				break;
 			case GTK_CELL_PIXTEXT:
 				gdk_window_get_size (GTK_CELL_PIXTEXT (clist_row->cell[i])->pixmap,
 									 &pixmap_width, &height);
-				width = (pixmap_width +
-						 GTK_CELL_PIXTEXT (clist_row->cell[i])->spacing +
-						 gdk_string_width (style->font,
-										   GTK_CELL_PIXTEXT
-										   (clist_row->cell[i])->text));
+				width += pixmap_width + GTK_CELL_PIXTEXT (clist_row->cell[i])->spacing;
 				break;
 			default:
-				continue;
 				break;
 		}
 
@@ -451,23 +454,21 @@ draw_row (GtkCList     *clist,
 									  (clip_rectangle.height - height) / 2,
 									  pixmap_width, height);
 				offset += GTK_CELL_PIXTEXT (clist_row->cell[i])->spacing;
-			case GTK_CELL_TEXT:
-				if (style != GTK_WIDGET (clist)->style)
-					row_center_offset = (((clist->row_height - style->font->ascent -
-										   style->font->descent - 1) / 2) + 1.5 +
-										 style->font->ascent);
-				else
-					row_center_offset = clist->row_center_offset;
 
-				gdk_gc_set_clip_rectangle (fg_gc, &clip_rectangle);
-				gdk_draw_string (clist->clist_window, style->font, fg_gc,
-								 offset,
-								 row_rectangle.y + row_center_offset + 
-								 clist_row->cell[i].vertical,
-								 (clist_row->cell[i].type == GTK_CELL_PIXTEXT) ?
-								 GTK_CELL_PIXTEXT (clist_row->cell[i])->text :
-								 GTK_CELL_TEXT (clist_row->cell[i])->text);
-				gdk_gc_set_clip_rectangle (fg_gc, NULL);
+				/* Fall through */
+			case GTK_CELL_TEXT:
+				if (layout)
+				{
+					gint row_center_offset = (clist->row_height - logical_rect.height - 1) / 2;
+
+					gdk_gc_set_clip_rectangle (fg_gc, &clip_rectangle);
+					gdk_draw_layout (clist->clist_window, fg_gc,
+									 offset,
+									 row_rectangle.y + row_center_offset + clist_row->cell[i].vertical,
+									 layout);
+					g_object_unref (G_OBJECT (layout));
+					gdk_gc_set_clip_rectangle (fg_gc, NULL);
+				}
 				break;
 			default:
 				break;
@@ -608,22 +609,27 @@ gnome_cmd_clist_get_type         (void)
 GtkWidget*
 gnome_cmd_clist_new (gint columns)
 {
-	GnomeCmdCList *clist;
-
-	clist = gtk_type_new (gnome_cmd_clist_get_type ());
-	gtk_clist_construct (GTK_CLIST (clist), columns, NULL);
-
-	return GTK_WIDGET (clist);
+	return gnome_cmd_clist_new_with_titles (columns, NULL);
 }
 
 
 GtkWidget*
 gnome_cmd_clist_new_with_titles (gint columns, gchar **titles)
 {
+	gint i;
 	GnomeCmdCList *clist;
 
-	clist = gtk_type_new (gnome_cmd_clist_get_type ());
-	gtk_clist_construct (GTK_CLIST (clist), columns, titles);
+	clist = g_object_new (gnome_cmd_clist_get_type(),
+						  "n_columns", columns,
+						  NULL);
+	
+	for ( i=0 ; i<columns ; i++ )
+		gtk_clist_set_column_auto_resize (GTK_CLIST (clist), i, TRUE);
+	
+	if (titles != NULL) {
+		for ( i=0 ; i<columns; i++ )
+			gtk_clist_set_column_title (GTK_CLIST (clist), i, titles[i]);
+	}
 
 	return GTK_WIDGET (clist);
 }
@@ -655,4 +661,17 @@ gnome_cmd_clist_set_voffset         (GnomeCmdCList *clist, gint voffset)
 }
 
 
+gint
+gnome_cmd_clist_get_row (GnomeCmdCList *clist, gint x, gint y)
+{
+	gint row;
+
+	g_return_val_if_fail (GNOME_CMD_IS_CLIST (clist), -1);
+	
+	if (gtk_clist_get_selection_info (GTK_CLIST (clist), x, y, &row, NULL) == 0)
+		row = -1;
+
+	return row;
+}
+
 
