@@ -22,11 +22,10 @@
 typedef struct _GnomeCmdData GnomeCmdData;
 typedef struct _GnomeCmdDataPrivate GnomeCmdDataPrivate;
 
-#include "gnome-cmd-ftp-server-types.h"
-#include "gnome-cmd-device.h"
 #include "gnome-cmd-app.h"
-#include "filter.h"
 #include "gnome-cmd-types.h"
+#include "gnome-cmd-con-list.h"
+#include "filter.h"
 #include "history.h"
 
 
@@ -35,13 +34,6 @@ struct _GnomeCmdData
 	GnomeCmdDataPrivate *priv;
 };
 
-typedef struct
-{
-	GdkColor *sel_fg, *sel_bg;
-	GdkColor *norm_fg, *norm_bg;
-	GdkColor *curs_fg, *curs_bg;
-} GnomeCmdLayoutColors;
-
 typedef enum
 {
 	RIGHT_BUTTON_POPUPS_MENU,
@@ -55,6 +47,7 @@ typedef struct
 	GList *directories;
 	gboolean recursive;
 	gboolean case_sens;	
+	gint width,height;
 } SearchDefaults;
 
 typedef struct
@@ -71,6 +64,17 @@ typedef struct
 	gint sep_value;
 } AdvrenameDefaults;
 
+typedef struct {
+	gboolean file_types[8];
+	
+	gboolean hidden;
+	gboolean backup;
+	gboolean other;
+	gchar *other_value;
+} FilterSettings;
+
+
+
 
 GnomeCmdData*
 gnome_cmd_data_new                       (void);
@@ -87,31 +91,8 @@ gnome_cmd_data_load                      (void);
 void
 gnome_cmd_data_load_more                 (void);
 
-void
-gnome_cmd_data_add_ftp_server            (GnomeCmdFtpServer *server);
-
-void
-gnome_cmd_data_remove_ftp_server         (GnomeCmdFtpServer *server);
-
-GList*
-gnome_cmd_data_get_ftp_servers           (void);
-
-void
-gnome_cmd_data_set_ftp_servers           (GList *servers);
-
-
-void
-gnome_cmd_data_add_device                (GnomeCmdDevice *device);
-
-void
-gnome_cmd_data_remove_device             (GnomeCmdDevice *device);
-
-GList*
-gnome_cmd_data_get_devices               (void);
-
-void
-gnome_cmd_data_set_devices               (GList *devices);
-
+GnomeCmdConList *
+gnome_cmd_data_get_con_list              (void);
 
 void
 gnome_cmd_data_add_fav_app                (GnomeCmdApp *app);
@@ -156,28 +137,42 @@ gnome_cmd_data_get_layout                (void);
 void
 gnome_cmd_data_set_layout                (GnomeCmdLayout layout);
 
+GnomeCmdColorMode
+gnome_cmd_data_get_color_mode            (void);
+
+void
+gnome_cmd_data_set_color_mode            (GnomeCmdColorMode mode);
+
+GnomeCmdColorTheme*
+gnome_cmd_data_get_current_color_theme   (void);
+
 gint
 gnome_cmd_data_get_list_row_height       (void);
 
 void
 gnome_cmd_data_set_list_row_height       (gint height);
 
-GnomeCmdLayoutColors*
-gnome_cmd_data_get_layout_colors    (void);
-
 void
-gnome_cmd_data_set_type_filter          (GnomeVFSFileType type,
-										 gboolean value);
+gnome_cmd_data_set_ext_disp_mode        (GnomeCmdExtDispMode mode);
+
+GnomeCmdExtDispMode
+gnome_cmd_data_get_ext_disp_mode        (void);
+
+FilterSettings *
+gnome_cmd_data_get_filter_settings      (void);
 
 gboolean
 gnome_cmd_data_get_type_filter          (GnomeVFSFileType type);
-
-void
-gnome_cmd_data_set_hidden_filter        (gboolean value);
-
+	
 gboolean
 gnome_cmd_data_get_hidden_filter        (void);
 
+gboolean
+gnome_cmd_data_get_backup_filter        (void);
+
+gboolean
+gnome_cmd_data_get_other_filter         (void);
+
 void
 gnome_cmd_data_set_position             (gint width, gint height);
 
@@ -211,18 +206,18 @@ gnome_cmd_data_get_case_sens_sort       (void);
 void
 gnome_cmd_data_set_case_sens_sort       (gboolean value);
 
+gboolean
+gnome_cmd_data_get_confirm_delete       (void);
+
+void
+gnome_cmd_data_set_confirm_delete       (gboolean value);
+
 const gchar *
 gnome_cmd_data_get_list_font             (void);
 
-const gchar *
-gnome_cmd_data_get_misc_font             (void);
-
 void
 gnome_cmd_data_set_list_font             (const gchar *list_font);
 
-void
-gnome_cmd_data_set_misc_font             (const gchar *misc_font);
-
 void
 gnome_cmd_data_set_right_mouse_button_mode (GnomeCmdRightMouseButtonMode mode);
 
@@ -316,16 +311,10 @@ gnome_cmd_data_set_use_ls_colors       (gboolean value);
 gboolean
 gnome_cmd_data_get_use_ls_colors       (void);
 
-void
-gnome_cmd_data_set_use_fam             (gboolean value);
-
-gboolean
-gnome_cmd_data_get_use_fam             (void);
-
 SearchDefaults *
 gnome_cmd_data_get_search_defaults     (void);
 
-GnomeCmdFtpServer *
+GnomeCmdConFtp *
 gnome_cmd_data_get_quick_connect_server (void);
 
 GnomeCmdBookmarkGroup *
@@ -361,11 +350,44 @@ gnome_cmd_data_set_buttonbar_visibility (gboolean value);
 AdvrenameDefaults *
 gnome_cmd_data_get_advrename_defaults   (void);
 
+gboolean
+gnome_cmd_data_get_list_orientation (void);
+
+void
+gnome_cmd_data_set_list_orientation (gboolean vertical);
+
+gboolean
+gnome_cmd_data_get_conbuttons_visibility (void);
+
+void
+gnome_cmd_data_set_conbuttons_visibility (gboolean value);
+
+gboolean
+gnome_cmd_data_get_cmdline_visibility (void);
+
+void
+gnome_cmd_data_set_cmdline_visibility (gboolean value);
+
+void
+gnome_cmd_data_set_start_dir (gboolean fs, const gchar *start_dir);
+
+const gchar *
+gnome_cmd_data_get_start_dir (gboolean fs);
+
 void
 gnome_cmd_data_set_last_pattern (const gchar *value);
 
 const gchar *
 gnome_cmd_data_get_last_pattern (void);
 
+GList *
+gnome_cmd_data_get_auto_load_plugins ();
+
+void
+gnome_cmd_data_set_auto_load_plugins (GList *plugins);
+
+guint
+gnome_cmd_data_get_gui_update_rate (void);
+
 
 #endif //__GNOME_CMD_DATA_H__
