@@ -19,8 +19,10 @@
 #include <config.h>
 #include "gnome-cmd-includes.h"
 #include <regex.h>
+#include "gnome-cmd-file-selector.h"
 #include "gnome-cmd-file-list.h"
-#include "gnome-cmd-file-funcs.h"
+#include "gnome-cmd-file.h"
+#include "gnome-cmd-main-win.h"
 #include "utils.h"
 #include "gnome-cmd-data.h"
 #include "gnome-cmd-patternsel-dialog.h"
@@ -29,7 +31,6 @@
 #include "gnome-cmd-style.h"
 #include "gnome-cmd-file-popmenu.h"
 #include "gnome-cmd-rename-dialog.h"
-#include "gnome-cmd-make-copy-dialog.h"
 #include "gnome-cmd-chown-dialog.h"
 #include "gnome-cmd-chmod-dialog.h"
 #include "gnome-cmd-delete-dialog.h"
@@ -123,6 +124,16 @@ struct _GnomeCmdFileListPrivate {
 };
 
 
+typedef struct
+{
+	gchar *text[FILE_LIST_NUM_COLUMNS];
+	
+	gchar *dpath;
+	gchar *fname;
+	gchar *fext;
+} FileFormatData;
+
+
 static int
 get_num_files (GnomeCmdFileList *fl)
 {
@@ -168,8 +179,6 @@ show_selpat_dialog (GnomeCmdFileList *fl, gboolean mode)
 	dialog = gnome_cmd_patternsel_dialog_new (fl, mode);
 
 	gtk_widget_ref (dialog);
-	gtk_object_set_data_full (GTK_OBJECT (fl), "gnome_cmd_patternsel_dialog", dialog,
-							  (GtkDestroyNotify)gtk_widget_unref);
 	gtk_signal_connect (GTK_OBJECT (dialog), "hide",
 						GTK_SIGNAL_FUNC (on_selpat_hide), fl);
 	gtk_widget_show (dialog);
@@ -197,8 +206,7 @@ on_quicksearch_popup_hide (GtkWidget *quicksearch_popup, GnomeCmdFileList *fl)
 
 
 static void
-do_select_file (GnomeCmdFileList *fl,
-				GnomeCmdFile *finfo)
+select_file (GnomeCmdFileList *fl, GnomeCmdFile *finfo)
 {
 	gint row;
 	
@@ -216,9 +224,11 @@ do_select_file (GnomeCmdFileList *fl,
 	if (!gnome_cmd_data_get_use_ls_colors ())
 		gtk_clist_set_row_style (GTK_CLIST (fl), row, sel_list_style);
 	else {
-		GnomeCmdLayoutColors *colors = gnome_cmd_data_get_layout_colors ();
-		gtk_clist_set_foreground (GTK_CLIST (fl), row, colors->sel_fg);
-		gtk_clist_set_background (GTK_CLIST (fl), row, colors->sel_bg);
+		GnomeCmdColorTheme *colors = gnome_cmd_data_get_current_color_theme ();
+		if (!colors->respect_theme) {
+			gtk_clist_set_foreground (GTK_CLIST (fl), row, colors->sel_fg);
+			gtk_clist_set_background (GTK_CLIST (fl), row, colors->sel_bg);
+		}
 	}
 
 	if (g_list_index (fl->priv->selected_files, finfo) != -1)
@@ -233,8 +243,7 @@ do_select_file (GnomeCmdFileList *fl,
 
 
 static void
-do_unselect_file (GnomeCmdFileList *fl,
-				  GnomeCmdFile *finfo)
+unselect_file (GnomeCmdFileList *fl, GnomeCmdFile *finfo)
 {
 	gint row;
 	
@@ -256,13 +265,13 @@ do_unselect_file (GnomeCmdFileList *fl,
 		gtk_clist_set_row_style (GTK_CLIST (fl), row, list_style);
 	else {
 		LsColor *col = ls_colors_get (finfo);
-		GnomeCmdLayoutColors *colors = gnome_cmd_data_get_layout_colors ();
+		GnomeCmdColorTheme *colors = gnome_cmd_data_get_current_color_theme ();
 		GdkColor *fg, *bg;
 		if (col) {
 			fg = col->fg;
 			bg = col->bg;
 			if (!fg) fg = colors->norm_fg;
-			if (!bg) bg = colors->norm_bg;			
+			if (!bg) bg = colors->norm_bg;
 			if (bg) gtk_clist_set_background (GTK_CLIST (fl), row, bg);
 			if (fg) gtk_clist_set_foreground (GTK_CLIST (fl), row, fg);
 		}
@@ -272,7 +281,7 @@ do_unselect_file (GnomeCmdFileList *fl,
 
 
 static void
-select_file (GnomeCmdFileList *fl,
+toggle_file (GnomeCmdFileList *fl,
 			 GnomeCmdFile *finfo)
 {
 	gint row;
@@ -287,16 +296,31 @@ select_file (GnomeCmdFileList *fl,
 	if (row < g_list_length (fl->priv->shown_files))
 	{
 		if (g_list_index (fl->priv->selected_files, finfo) == -1)
-			do_select_file (fl, finfo);
+			select_file (fl, finfo);
 		else
-			do_unselect_file (fl, finfo);
+			unselect_file (fl, finfo);
 	}
 }
 
 
 static void
-select_file_at_row (GnomeCmdFileList *fl,
-					gint row)
+toggle_file_at_row (GnomeCmdFileList *fl, gint row)
+{
+	GnomeCmdFile *finfo;
+
+	g_return_if_fail (GNOME_CMD_IS_FILE_LIST (fl));
+
+	fl->priv->cur_file = row;
+
+	finfo = get_file_at_row (fl, row);
+	if (!finfo) return;
+
+	toggle_file (fl, finfo);
+}
+
+
+static void
+select_file_at_row (GnomeCmdFileList *fl, gint row)
 {
 	GnomeCmdFile *finfo;
 
@@ -325,12 +349,32 @@ select_file_range (GnomeCmdFileList *fl,
 		start_row = end_row;
 		end_row = i;
 	}
-        
+
 	for ( i=start_row ; i<end_row+1 ; i++ )
 		select_file_at_row (fl, i);
 }
 
 
+static void
+toggle_file_range (GnomeCmdFileList *fl,
+				   gint start_row, gint end_row)
+{
+	gint i;
+
+	g_return_if_fail (GNOME_CMD_IS_FILE_LIST (fl));
+
+	if (start_row > end_row)
+	{
+		i = start_row;
+		start_row = end_row;
+		end_row = i;
+	}
+
+	for ( i=start_row ; i<end_row+1 ; i++ )
+		toggle_file_at_row (fl, i);
+}
+
+
 static void toggle_files_with_same_extension (GnomeCmdFileList *fl,
 											  gboolean select)
 {
@@ -358,9 +402,9 @@ static void toggle_files_with_same_extension (GnomeCmdFileList *fl,
 
 			if (ext2 && strcmp (ext1, ext2) == 0) {
 				if (select)
-					do_select_file (fl, finfo);
+					select_file (fl, finfo);
 				else
-					do_unselect_file (fl, finfo);
+					unselect_file (fl, finfo);
 			}
 		}
 			
@@ -391,9 +435,9 @@ toggle_with_pattern (GnomeCmdFileList *fl, const gchar *pattern, gboolean case_s
 		if (finfo && finfo->info) {
 			if (filter_match (filter, finfo->info->name)) {
 				if (mode)
-					do_select_file (fl, finfo);
+					select_file (fl, finfo);
 				else
-					do_unselect_file (fl, finfo);
+					unselect_file (fl, finfo);
 			}
 		}
 			
@@ -432,7 +476,6 @@ create_column_titles (GnomeCmdFileList *fl)
 								  (GtkDestroyNotify) gtk_widget_unref);
 		gtk_widget_show (fl->priv->column_labels[i]);
 		gtk_box_pack_start (GTK_BOX (hbox), fl->priv->column_labels[i], TRUE, TRUE, 0);
-		gtk_widget_modify_style (fl->priv->column_labels[i], main_rc_style);
 
 		pixmap = gtk_pixmap_new (pm, bm);
 		gtk_widget_ref (pixmap);
@@ -443,10 +486,13 @@ create_column_titles (GnomeCmdFileList *fl)
 		
 		fl->priv->column_pixmaps[i] = pixmap;
 		gtk_clist_set_column_widget (GTK_CLIST (fl), i, hbox);
+	}
+
+	for ( i=1 ; i<FILE_LIST_NUM_COLUMNS ; i++ ) {
 		gtk_clist_set_column_width (GTK_CLIST (fl), i, gnome_cmd_data_get_fs_col_width (i));
 		gtk_clist_set_column_justification (GTK_CLIST (fl), i, list_column_justification[i]);
 	}
-
+	
 	gtk_clist_column_titles_show (GTK_CLIST (fl));	
 }
 
@@ -571,6 +617,8 @@ drag_data_get (GtkWidget        *widget,
 
 	data = build_selected_file_list (fl, &len);
 
+	if (!data) return;
+	
 	switch (info){
 		case TARGET_URI_LIST:
 		case TARGET_TEXT_PLAIN:
@@ -578,7 +626,7 @@ drag_data_get (GtkWidget        *widget,
 			break;
 
 		case TARGET_URL:
-			files = gnome_uri_list_extract_uris (data);
+			files = gnome_vfs_uri_list_parse (data);
 			if (files) {
 				gtk_selection_data_set (selection_data,
 										selection_data->target,
@@ -586,7 +634,7 @@ drag_data_get (GtkWidget        *widget,
 										files->data,
 										strlen (files->data));
 			}
-			gnome_uri_list_free_strings (files);
+			g_list_foreach (files, (GFunc)g_free, NULL);
 			break;
 
 		default:
@@ -646,7 +694,7 @@ show_file_popup (GnomeCmdFileList *fl, GdkEventButton *event)
 							  "file_popup_menu", menu,
 							  (GtkDestroyNotify)gtk_widget_unref);
 
-	gnome_popup_menu_do_popup (menu, NULL, fl, event, fl);
+	gnome_popup_menu_do_popup (menu, NULL, fl, event, fl, NULL);
 }
 
 
@@ -670,7 +718,7 @@ on_right_mb_timeout (GnomeCmdFileList *fl)
 	
 	if (fl->priv->right_mb_down_file == focus_file) {
 		show_file_popup (fl, NULL);
-		select_file (fl, focus_file);
+		toggle_file (fl, focus_file);
 		return FALSE;
 	}	
 
@@ -769,20 +817,31 @@ sort_by_extension (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl)
 static gint
 sort_by_dir (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl)
 {
+	gint ret = 0;
 	gboolean raising = fl->priv->sort_raising[fl->priv->current_col];
 
-	const gchar *d1 = gnome_cmd_file_get_dir_path (f1);
-	const gchar *d2 = gnome_cmd_file_get_dir_path (f2);
+	gchar *t1 = gnome_cmd_file_get_path (f1);
+	gchar *t2 = gnome_cmd_file_get_path (f2);
+	gchar *d1 = g_dirname (t1);
+	gchar *d2 = g_dirname (t2);
 
 	if (strcmp (f1->info->name, "..") == 0)
-		return -1;
+		ret = -1;
 	else if (strcmp (f2->info->name, "..") == 0)
-		return 1;
+		ret = 1;
 	else if (f1->info->type > f2->info->type)
-		return -1;
+		ret = -1;
 	else if (f1->info->type < f2->info->type)
-		return 1;
+		ret = 1;
 
+	g_free (t1);
+	g_free (t2);
+	g_free (d1);
+	g_free (d2);
+
+	if (ret)
+		return ret;
+	
 	return my_strcmp (d1, d2, raising);
 }
 
@@ -987,20 +1046,17 @@ on_scroll_vertical                  (GtkCList        *clist,
 		if (start_row < 0 || end_row < 0)
 			return;
         
-		if (end_row < num_files-1)
-			end_row--;
-		
         if (scroll_type == GTK_SCROLL_STEP_BACKWARD || scroll_type == GTK_SCROLL_STEP_FORWARD)
-			select_file_at_row (fl, start_row);
+			toggle_file_at_row (fl, start_row);
         else
-			select_file_range (fl, start_row, end_row);
+			toggle_file_range (fl, start_row, end_row);
     }
 
 	fl->priv->cur_file = clist->focus_row;
 }
 							   
 
-static gint
+static gboolean
 on_button_press (GtkCList *clist, GdkEventButton *event, GnomeCmdFileList *fl)
 {
 	gint row;
@@ -1016,7 +1072,7 @@ on_button_press (GtkCList *clist, GdkEventButton *event, GnomeCmdFileList *fl)
 	gtk_signal_emit (
 		GTK_OBJECT (fl), file_list_signals[LIST_CLICKED], event);
 	
-	row = gnome_cmd_file_list_get_row (fl, event->x, event->y);
+	row = gnome_cmd_clist_get_row (GNOME_CMD_CLIST (fl), event->x, event->y);
 	if (row < 0) {
 		gtk_signal_emit (
 			GTK_OBJECT (fl), file_list_signals[EMPTY_SPACE_CLICKED], event);
@@ -1074,17 +1130,17 @@ on_file_clicked (GnomeCmdFileList *fl,
 			if (event->state & GDK_SHIFT_MASK)
 				select_file_range (fl, fl->priv->shift_down_row, row);
 			else if (event->state & GDK_CONTROL_MASK)
-				select_file_at_row (fl, row);
+				toggle_file_at_row (fl, row);
 		}
 		else if (event->button == 3) {
 			if (strcmp (finfo->info->name, "..") != 0) {
 				if (gnome_cmd_data_get_right_mouse_button_mode() == RIGHT_BUTTON_SELECTS) {
 					if (g_list_index (fl->priv->selected_files, finfo) == -1) {
-						do_select_file (fl, finfo);
+						select_file (fl, finfo);
 						fl->priv->right_mouse_sel_state = 1;
 					}
 					else {
-						do_unselect_file (fl, finfo);
+						unselect_file (fl, finfo);
 						fl->priv->right_mouse_sel_state = 0;
 					}
 
@@ -1116,16 +1172,16 @@ on_motion_notify                     (GtkCList *clist,
 		y = event->y;
 		y -= (clist->column_title_area.height - GTK_CONTAINER (clist)->border_width);
 				
-		row = gnome_cmd_file_list_get_row (fl, event->x, y);
+		row = gnome_cmd_clist_get_row (GNOME_CMD_CLIST (fl), event->x, y);
 		
 		if (row != -1) {
 			GnomeCmdFile *finfo = gnome_cmd_file_list_get_file_at_row (fl, row+1);
 			if (finfo) {
 				gnome_cmd_file_list_select_row (fl, row+1);
 				if (fl->priv->right_mouse_sel_state)
-					do_select_file (fl, finfo);
+					select_file (fl, finfo);
 				else
-					do_unselect_file (fl, finfo);
+					unselect_file (fl, finfo);
 			}
 		}		
 	}
@@ -1144,7 +1200,7 @@ on_button_release (GtkWidget *widget, GdkEventButton *event, GnomeCmdFileList *f
 	if (GTK_CLIST (fl)->clist_window != event->window)
 		return FALSE;
 
-	row = gnome_cmd_file_list_get_row (fl, event->x, event->y);
+	row = gnome_cmd_clist_get_row (GNOME_CMD_CLIST (fl), event->x, event->y);
 	if (row < 0)
 		return FALSE;
 
@@ -1171,7 +1227,6 @@ on_realize                               (GnomeCmdFileList *fl,
 {
 	g_return_if_fail (GNOME_CMD_IS_FILE_LIST (fl));
 
-	create_column_titles (fl);
 	update_column_sort_arrows (fl);
 }
 
@@ -1217,7 +1272,7 @@ class_init (GnomeCmdFileListClass *class)
 	file_list_signals[FILE_CLICKED] =
 		gtk_signal_new ("file_clicked",
 			GTK_RUN_LAST,
-		    object_class->type,
+		    G_OBJECT_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GnomeCmdFileListClass, file_clicked),
 		    gtk_marshal_NONE__POINTER_POINTER,
 		    GTK_TYPE_NONE,
@@ -1226,7 +1281,7 @@ class_init (GnomeCmdFileListClass *class)
 	file_list_signals[LIST_CLICKED] =
 		gtk_signal_new ("list_clicked",
 			GTK_RUN_LAST,
-		    object_class->type,
+		    G_OBJECT_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GnomeCmdFileListClass, list_clicked),
 		    gtk_marshal_NONE__POINTER,
 		    GTK_TYPE_NONE,
@@ -1235,7 +1290,7 @@ class_init (GnomeCmdFileListClass *class)
 	file_list_signals[EMPTY_SPACE_CLICKED] =
 		gtk_signal_new ("empty_space_clicked",
 			GTK_RUN_LAST,
-		    object_class->type,
+		    G_OBJECT_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GnomeCmdFileListClass, empty_space_clicked),
 		    gtk_marshal_NONE__POINTER,
 		    GTK_TYPE_NONE,
@@ -1244,14 +1299,12 @@ class_init (GnomeCmdFileListClass *class)
 	file_list_signals[SELECTION_CHANGED] =
 		gtk_signal_new ("selection_changed",
 			GTK_RUN_LAST,
-		    object_class->type,
+		    G_OBJECT_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GnomeCmdFileListClass, selection_changed),
 		    gtk_marshal_NONE__NONE,
 		    GTK_TYPE_NONE,
 			0);
 	
-	gtk_object_class_add_signals (object_class, file_list_signals, LAST_SIGNAL);
-	
 	object_class->destroy = destroy;
 	widget_class->map = map;	
 	class->file_clicked = NULL;
@@ -1288,10 +1341,11 @@ init (GnomeCmdFileList *fl)
 	for ( i=0 ; i<FILE_LIST_NUM_COLUMNS ; i++ )
 		fl->priv->sort_raising[i] = FALSE;
 
-	gtk_clist_construct (GTK_CLIST (fl), FILE_LIST_NUM_COLUMNS, list_column_titles);
-	
 	init_dnd (fl);
 	
+	for ( i=0 ; i<FILE_LIST_NUM_COLUMNS ; i++ )
+		gtk_clist_set_column_resizeable (GTK_CLIST (fl), i, TRUE);
+	
 	gtk_signal_connect_after (GTK_OBJECT (fl), "scroll_vertical",
 							  GTK_SIGNAL_FUNC (on_scroll_vertical), fl);
 	gtk_signal_connect (GTK_OBJECT (fl), "click-column",
@@ -1304,7 +1358,7 @@ init (GnomeCmdFileList *fl)
 	gtk_signal_connect (GTK_OBJECT (fl), "motion-notify-event",
 						GTK_SIGNAL_FUNC (on_motion_notify), fl);
 	
-	gtk_signal_connect (GTK_OBJECT (fl), "realize",
+	gtk_signal_connect_after (GTK_OBJECT (fl), "realize",
 						GTK_SIGNAL_FUNC (on_realize), fl);
 	gtk_signal_connect (GTK_OBJECT (fl), "file-clicked",
 						GTK_SIGNAL_FUNC (on_file_clicked), fl);
@@ -1346,8 +1400,12 @@ gnome_cmd_file_list_new ()
 {
 	GnomeCmdFileList *fl;
 
-	fl = gtk_type_new (gnome_cmd_file_list_get_type ());
+	fl = g_object_new (gnome_cmd_file_list_get_type(),
+					   "n_columns", FILE_LIST_NUM_COLUMNS,
+					   NULL);
 
+	create_column_titles (fl);
+	
 	return GTK_WIDGET (fl);
 }
 
@@ -1373,6 +1431,84 @@ gnome_cmd_file_list_update_style (GnomeCmdFileList *fl)
 }
 
 
+static gchar *
+strip_extension (const gchar *fname)
+{
+	gchar *s = g_strdup (fname);
+	gchar *p = g_strrstr (s, ".");
+	if (p && p != s)		
+		*p = '\0';
+	return s;
+}
+
+
+static void
+format_file_for_display (GnomeCmdFile *finfo, FileFormatData *data, gboolean tree_size)
+{
+	gchar *t, *t2;
+	gchar *size;
+
+	
+	/* If the user wants a character insted of icon for filetype set it now
+	 *
+	 */
+	if (gnome_cmd_data_get_layout () == GNOME_CMD_LAYOUT_TEXT)
+		data->text[FILE_LIST_COLUMN_ICON] = (gchar*)gnome_cmd_file_get_type_string (finfo);
+	else
+		data->text[FILE_LIST_COLUMN_ICON] = NULL;
+
+	/* Prepare the strings to show
+	 *
+	 */
+	t = gnome_cmd_file_get_path (finfo);
+	t2 = g_dirname (t);
+	g_free (t);
+	data->dpath = get_utf8 (t2);
+	g_free (t2);
+	
+	if (gnome_cmd_data_get_ext_disp_mode() == GNOME_CMD_EXT_DISP_STRIPPED
+		&& finfo->info->type == GNOME_VFS_FILE_TYPE_REGULAR) {
+		t = strip_extension (gnome_cmd_file_get_name (finfo));
+		data->fname = get_utf8 (t);
+		g_free (t);
+	}	
+	else
+		data->fname = get_utf8 (gnome_cmd_file_get_name (finfo) );
+
+	if (gnome_cmd_data_get_ext_disp_mode() != GNOME_CMD_EXT_DISP_WITH_FNAME)
+		data->fext = get_utf8 (gnome_cmd_file_get_extension (finfo));
+	else
+		data->fext = NULL;
+
+	if (tree_size)
+		size = (gchar*)gnome_cmd_file_get_tree_size (finfo);
+	else
+		size = (gchar*)gnome_cmd_file_get_size (finfo);
+	
+	/* Set other file information
+	 *
+	 */
+	data->text[FILE_LIST_COLUMN_NAME]  = data->fname;
+ 	data->text[FILE_LIST_COLUMN_EXT]   = data->fext;
+	data->text[FILE_LIST_COLUMN_DIR]   = data->dpath;
+	data->text[FILE_LIST_COLUMN_SIZE]  = size;
+	data->text[FILE_LIST_COLUMN_DATE]  = (gchar*)gnome_cmd_file_get_mdate (finfo, FALSE);
+	data->text[FILE_LIST_COLUMN_PERM]  = (gchar*)gnome_cmd_file_get_perm (finfo);
+	data->text[FILE_LIST_COLUMN_OWNER] = (gchar*)gnome_cmd_file_get_owner (finfo);
+	data->text[FILE_LIST_COLUMN_GROUP] = (gchar*)gnome_cmd_file_get_group (finfo);
+	data->text[FILE_LIST_NUM_COLUMNS]  = NULL;
+}
+
+
+static void
+cleanup_file_format (FileFormatData *data)
+{
+	g_free (data->dpath);
+	g_free (data->fname);
+	if (data->fext)
+		g_free (data->fext);
+}
+
 
 /******************************************************************************
 *
@@ -1393,31 +1529,13 @@ void
 gnome_cmd_file_list_add_file (GnomeCmdFileList *fl, GnomeCmdFile *finfo, gint in_row)
 {
 	gint row;
-	gchar *text[10];
+	FileFormatData data;
 	GtkCList *clist;
 
 	clist = GTK_CLIST (fl);
 
-	/* If the user wants a character insted of icon for filetype set it now
-	 *
-	 */
-	if (gnome_cmd_data_get_layout () == GNOME_CMD_LAYOUT_TEXT)
-		text[FILE_LIST_COLUMN_ICON] = (gchar*)gnome_cmd_file_get_type_string (finfo);
-	else
-		text[FILE_LIST_COLUMN_ICON] = NULL;
-
-	/* Set other file information
-	 *
-	 */
-	text[FILE_LIST_COLUMN_NAME]  = (gchar*)gnome_cmd_file_get_name (finfo);
-	text[FILE_LIST_COLUMN_EXT]   = (gchar*)gnome_cmd_file_get_extension (finfo);
-	text[FILE_LIST_COLUMN_DIR]   = (gchar*)gnome_cmd_file_get_dir_path (finfo);
-	text[FILE_LIST_COLUMN_SIZE]  = (gchar*)gnome_cmd_file_get_size (finfo);
-	text[FILE_LIST_COLUMN_DATE]  = (gchar*)gnome_cmd_file_get_mdate (finfo, FALSE);
-	text[FILE_LIST_COLUMN_PERM]  = (gchar*)gnome_cmd_file_get_perm (finfo);
-	text[FILE_LIST_COLUMN_OWNER] = (gchar*)gnome_cmd_file_get_owner (finfo);
-	text[FILE_LIST_COLUMN_GROUP] = (gchar*)gnome_cmd_file_get_group (finfo);
-	text[FILE_LIST_NUM_COLUMNS]  = NULL;
+	format_file_for_display (finfo, &data, FALSE);
+	
 
 	/* Add the file to the list
 	 *
@@ -1425,9 +1543,12 @@ gnome_cmd_file_list_add_file (GnomeCmdFileList *fl, GnomeCmdFile *finfo, gint in
 	gnome_cmd_file_ref (finfo);
 	fl->priv->shown_files = g_list_append (fl->priv->shown_files, finfo);	
 	if (in_row == -1)
-		row = gtk_clist_append (clist, text);	
+		row = gtk_clist_append (clist, data.text);	
 	else
-		row = gtk_clist_insert (clist, in_row, text);	
+		row = gtk_clist_insert (clist, in_row, data.text);
+
+	cleanup_file_format (&data);
+
 	
 	/* Setup row data and color
 	 *
@@ -1525,25 +1646,44 @@ gnome_cmd_file_list_insert_file (GnomeCmdFileList *fl, GnomeCmdFile *finfo)
 void
 gnome_cmd_file_list_update_file (GnomeCmdFileList *fl, GnomeCmdFile *finfo)
 {
-	gchar *text[10];
+	FileFormatData data;
 	gint i,row;
 
+	if (!gnome_cmd_file_needs_update (finfo))
+		return;
+	
 	row = get_row_from_file (fl, finfo);
 	if (row == -1)
 		return;
-	
-	text[FILE_LIST_COLUMN_NAME]  = (gchar*)gnome_cmd_file_get_name (finfo);
-	text[FILE_LIST_COLUMN_EXT]   = (gchar*)gnome_cmd_file_get_extension (finfo);
-	text[FILE_LIST_COLUMN_DIR]   = (gchar*)gnome_cmd_file_get_dir_path (finfo);
-	text[FILE_LIST_COLUMN_SIZE]  = (gchar*)gnome_cmd_file_get_size (finfo);
-	text[FILE_LIST_COLUMN_DATE]  = (gchar*)gnome_cmd_file_get_mdate (finfo, FALSE);
-	text[FILE_LIST_COLUMN_PERM]  = (gchar*)gnome_cmd_file_get_perm (finfo);
-	text[FILE_LIST_COLUMN_OWNER] = (gchar*)gnome_cmd_file_get_owner (finfo);
-	text[FILE_LIST_COLUMN_GROUP] = (gchar*)gnome_cmd_file_get_group (finfo);
-	text[FILE_LIST_NUM_COLUMNS]  = NULL;
+
+	format_file_for_display (finfo, &data, FALSE);
 
 	for ( i=1 ; i<FILE_LIST_NUM_COLUMNS; i++ )
-		gtk_clist_set_text (GTK_CLIST (fl), row, i, text[i]);
+		gtk_clist_set_text (GTK_CLIST (fl), row, i, data.text[i]);
+
+	cleanup_file_format (&data);
+}
+
+
+void
+gnome_cmd_file_list_show_dir_size (GnomeCmdFileList *fl, GnomeCmdFile *finfo)
+{
+	gint i,row;
+	FileFormatData data;
+	
+	g_return_if_fail (GNOME_CMD_IS_FILE_LIST (fl));
+	g_return_if_fail (GNOME_CMD_IS_FILE (finfo));
+	
+	row = get_row_from_file (fl, finfo);
+	if (row == -1)
+		return;
+
+	format_file_for_display (finfo, &data, TRUE);
+	
+	for ( i=1 ; i<FILE_LIST_NUM_COLUMNS; i++ )
+		gtk_clist_set_text (GTK_CLIST (fl), row, i, data.text[i]);
+
+	cleanup_file_format (&data);
 }
 
 
@@ -1556,12 +1696,14 @@ gnome_cmd_file_list_remove_file (GnomeCmdFileList *fl, GnomeCmdFile *finfo)
 	g_return_if_fail (finfo != NULL);
 	
 	row = get_row_from_file (fl, finfo);
-	gtk_clist_remove (GTK_CLIST (fl), row);
-	fl->priv->selected_files = g_list_remove (fl->priv->selected_files, finfo);
-	fl->priv->shown_files = g_list_remove (fl->priv->shown_files, finfo);
-	gnome_cmd_file_unref (finfo);
+	if (row >= 0) {
+		gtk_clist_remove (GTK_CLIST (fl), row);
+		fl->priv->selected_files = g_list_remove (fl->priv->selected_files, finfo);
+		fl->priv->shown_files = g_list_remove (fl->priv->shown_files, finfo);
+		gnome_cmd_file_unref (finfo);
 
-	focus_file_at_row (fl, MIN(row, GTK_CLIST(fl)->focus_row));
+		focus_file_at_row (fl, MIN(row, GTK_CLIST(fl)->focus_row));
+	}
 }
 
 
@@ -1753,7 +1895,7 @@ gnome_cmd_file_list_select_all (GnomeCmdFileList *fl)
 	tmp = gnome_cmd_file_list_get_all_files (fl);
 	while (tmp) {
 		GnomeCmdFile *finfo = (GnomeCmdFile*)tmp->data;
-		do_select_file (fl, finfo);
+		select_file (fl, finfo);
 		tmp = tmp->next;
 	}
 }
@@ -1769,7 +1911,7 @@ gnome_cmd_file_list_unselect_all (GnomeCmdFileList *fl)
 	tmp = g_list_copy (fl->priv->selected_files);
 	while (tmp)	{
 		GnomeCmdFile *finfo = (GnomeCmdFile*)tmp->data;
-		do_unselect_file (fl, finfo);
+		unselect_file (fl, finfo);
 		tmp = tmp->next;
 	}
 
@@ -1788,7 +1930,7 @@ gnome_cmd_file_list_toggle (GnomeCmdFileList *fl)
 
 	finfo = get_file_at_row (fl, fl->priv->cur_file);
 	if (finfo)
-		select_file (fl, finfo);
+		toggle_file (fl, finfo);
 }
 
 
@@ -1801,7 +1943,7 @@ gnome_cmd_file_list_toggle_and_step (GnomeCmdFileList *fl)
 
 	finfo = get_file_at_row (fl, fl->priv->cur_file);
 	if (finfo)
-		select_file (fl, finfo);
+		toggle_file (fl, finfo);
 	if (fl->priv->cur_file < get_num_files (fl)-1)
 		focus_file_at_row (fl, fl->priv->cur_file+1);
 }
@@ -1842,20 +1984,6 @@ gnome_cmd_file_list_focus_file (GnomeCmdFileList *fl,
 }
 
 
-gint
-gnome_cmd_file_list_get_row (GnomeCmdFileList *fl, gint x, gint y)
-{
-	gint row;
-
-	g_return_val_if_fail (GNOME_CMD_IS_FILE_LIST (fl), -1);
-	
-	if (gtk_clist_get_selection_info (GTK_CLIST (fl), x, y, &row, NULL) == 0)
-		row = -1;
-
-	return row;
-}
-
-
 void
 gnome_cmd_file_list_select_row (GnomeCmdFileList *fl, gint row)
 {
@@ -1897,9 +2025,9 @@ gnome_cmd_file_list_invert_selection (GnomeCmdFileList *fl)
 
 		if (finfo && finfo->info) {
 			if (g_list_index (sel, finfo) == -1)
-				do_select_file (fl, finfo);
+				select_file (fl, finfo);
 			else
-				do_unselect_file (fl, finfo);			
+				unselect_file (fl, finfo);			
 		}
 			
 		tmp = tmp->next;
@@ -1929,6 +2057,65 @@ gnome_cmd_file_list_restore_selection (GnomeCmdFileList *fl)
 }
 
 
+static gint 
+compare_filename(GnomeCmdFile *f1, GnomeCmdFile *f2)
+{
+	return strcmp(f1->info->name, f2->info->name);
+}
+
+
+void
+gnome_cmd_file_list_compare_directories (void)
+{
+	GnomeCmdFileSelector *fs1 = gnome_cmd_main_win_get_active_fs (main_win);
+	GnomeCmdFileList     *fl1 = fs1 ? fs1->list : NULL;
+	GnomeCmdFileSelector *fs2 = gnome_cmd_main_win_get_inactive_fs (main_win);
+	GnomeCmdFileList     *fl2 = fs2 ? fs2->list : NULL;
+
+	GList *i;
+	
+	g_return_if_fail (GNOME_CMD_IS_FILE_LIST (fl1) || GNOME_CMD_IS_FILE_LIST (fl2));
+	
+	gnome_cmd_file_list_unselect_all (fl1);
+	gnome_cmd_file_list_select_all (fl2);
+	
+	for (i=gnome_cmd_file_list_get_all_files(fl1); i; i=i->next)
+	{
+		GnomeCmdFile *f1 = (GnomeCmdFile*)i->data;
+		GnomeCmdFile *f2;
+		GList *gl2 = g_list_find_custom (fl2->priv->selected_files, f1, (GCompareFunc)compare_filename);
+		
+		if (!gl2)
+		{
+			select_file (fl1, f1);
+			continue;
+		}
+		
+		f2 = (GnomeCmdFile*)gl2->data;
+		
+		if (f1->info->type==GNOME_VFS_FILE_TYPE_DIRECTORY || f2->info->type==GNOME_VFS_FILE_TYPE_DIRECTORY)
+		{
+			unselect_file (fl2, f2);
+			continue;	
+		}
+		
+		if (f1->info->mtime > f2->info->mtime)
+		{
+			select_file (fl1, f1);
+			unselect_file (fl2, f2);
+			continue;	
+		}
+		
+		if (f1->info->mtime == f2->info->mtime) {
+			if (f1->info->size == f2->info->size)
+				unselect_file (fl2, f2);
+			else
+				select_file (fl1, f1);
+		}
+	}
+}
+
+
 void
 gnome_cmd_file_list_sort (GnomeCmdFileList *fl)
 {
@@ -1971,7 +2158,7 @@ gnome_cmd_file_list_sort (GnomeCmdFileList *fl)
 		GnomeCmdFile *finfo;
 		
 		finfo = GNOME_CMD_FILE (list->data);
-		do_select_file (fl, finfo);
+		select_file (fl, finfo);
 		list = list->next;
 	}
 	
@@ -1993,26 +2180,6 @@ gnome_cmd_file_list_get_row_from_file (GnomeCmdFileList *fl, GnomeCmdFile *finfo
 }
 
 
-void
-gnome_cmd_file_list_show_make_copy_dialog (GnomeCmdFileList *fl)
-{
-	GnomeCmdFile *finfo;
-
-	g_return_if_fail (GNOME_CMD_IS_FILE_LIST (fl));
-
-	finfo = gnome_cmd_file_list_get_selected_file (fl);
-
-	if (GNOME_CMD_IS_FILE (finfo)) {
-		GtkWidget *dialog = gnome_cmd_make_copy_dialog_new (finfo);
-		
-		gtk_widget_ref (dialog);
-		gtk_object_set_data_full (GTK_OBJECT (fl), "gnome_cmd_make_copy_dialog", dialog,
-								  (GtkDestroyNotify)gtk_widget_unref);
-		gtk_widget_show (dialog);
-	}
-}
-
-
 void
 gnome_cmd_file_list_show_rename_dialog (GnomeCmdFileList *fl)
 {
@@ -2026,8 +2193,6 @@ gnome_cmd_file_list_show_rename_dialog (GnomeCmdFileList *fl)
 		GtkWidget *dialog = gnome_cmd_rename_dialog_new (finfo);
 		
 		gtk_widget_ref (dialog);
-		gtk_object_set_data_full (GTK_OBJECT (fl), "gnome_cmd_rename_dialog", dialog,
-								  (GtkDestroyNotify)gtk_widget_unref);
 		gtk_widget_show (dialog);
 	}
 }
@@ -2063,8 +2228,6 @@ gnome_cmd_file_list_show_chown_dialog (GnomeCmdFileList *fl)
 	
 		dialog = gnome_cmd_chown_dialog_new (files);
 		gtk_widget_ref (dialog);
-		gtk_object_set_data_full (GTK_OBJECT (fl), "gnome_cmd_chown_dialog", dialog,
-								  (GtkDestroyNotify)gtk_widget_unref);
 		gtk_widget_show (dialog);
 		g_list_free (files);
 	}
@@ -2085,8 +2248,6 @@ gnome_cmd_file_list_show_chmod_dialog (GnomeCmdFileList *fl)
 	
 		dialog = gnome_cmd_chmod_dialog_new (files);
 		gtk_widget_ref (dialog);
-		gtk_object_set_data_full (GTK_OBJECT (fl), "gnome_cmd_chmod_dialog", dialog,
-								  (GtkDestroyNotify)gtk_widget_unref);
 		gtk_widget_show (dialog);
 		g_list_free (files);
 	}
@@ -2106,8 +2267,6 @@ gnome_cmd_file_list_show_advrename_dialog (GnomeCmdFileList *fl)
 		GtkWidget *dialog = gnome_cmd_advrename_dialog_new (files);
 
 		gtk_widget_ref (dialog);
-		gtk_object_set_data_full (GTK_OBJECT (fl), "gnome_cmd_advrename_dialog", dialog,
-								  (GtkDestroyNotify)gtk_widget_unref);
 		gtk_widget_show (dialog);
 	}
 }
@@ -2212,6 +2371,7 @@ gnome_cmd_file_list_show_quicksearch (GnomeCmdFileList *fl, gchar c)
 		GnomeCmdQuicksearchPopup *popup =
 			GNOME_CMD_QUICKSEARCH_POPUP (fl->priv->quicksearch_popup);
 		gtk_entry_set_text (GTK_ENTRY (popup->entry), text);
+		gtk_entry_set_position (GTK_ENTRY (popup->entry), 1);
 	}
 
 	gtk_signal_connect (GTK_OBJECT (fl->priv->quicksearch_popup), "hide",
@@ -2224,7 +2384,7 @@ gnome_cmd_file_list_keypressed (GnomeCmdFileList *fl,
 								GdkEventKey *event)
 {
 	gint num_files;
-	
+
 	g_return_val_if_fail (event != NULL, FALSE);
 	g_return_val_if_fail (GNOME_CMD_IS_FILE_LIST (fl), FALSE);
 
@@ -2244,10 +2404,6 @@ gnome_cmd_file_list_keypressed (GnomeCmdFileList *fl,
 			case GDK_KP_Subtract:
 				toggle_files_with_same_extension (fl, FALSE);
 				break;
-
-			case GDK_b: //FIXME: Remove this
-				g_printerr ("voffset: %d\n", GTK_CLIST(fl)->voffset);
-				break;
 		}
 	}
 	else if (state_is_ctrl_alt (event->state)) {
@@ -2258,65 +2414,60 @@ gnome_cmd_file_list_keypressed (GnomeCmdFileList *fl,
 	else if (state_is_shift (event->state)) {
 		switch (event->keyval)
 		{
-			case GDK_F5:
-				gnome_cmd_file_list_show_make_copy_dialog (fl);
+			case GDK_F2:
+				gnome_cmd_file_list_compare_directories ();
 				return TRUE;
 				
 			case GDK_F6:
 				gnome_cmd_file_list_show_rename_dialog (fl);
 				return TRUE;
-
+				
 			case GDK_Left:
 			case GDK_Right:
-				//FIXME: This doesn't work as expected
 				event->state -= GDK_SHIFT_MASK;
 				return FALSE;
 
 			case GDK_Page_Up:
 			case GDK_KP_Page_Up:
 				fl->priv->shift_down = TRUE;
-				event->state -= GDK_SHIFT_MASK;
-				event->keyval = GDK_Page_Up;
+				gtk_signal_emit_by_name (
+					GTK_OBJECT (fl), "scroll_vertical", GTK_SCROLL_PAGE_BACKWARD, 0.0, NULL);
 				return FALSE;
 
 			case GDK_Page_Down:
 			case GDK_KP_Page_Down:
 				fl->priv->shift_down = TRUE;
-				event->state -= GDK_SHIFT_MASK;
-				event->keyval = GDK_Page_Down;
+				gtk_signal_emit_by_name (
+					GTK_OBJECT (fl), "scroll_vertical", GTK_SCROLL_PAGE_FORWARD, 0.0, NULL);
 				return FALSE;
 
 			case GDK_Up:	
 			case GDK_KP_Up:
 				fl->priv->shift_down = TRUE;
-				event->state -= GDK_SHIFT_MASK;
-				event->keyval = GDK_Up;
+				gtk_signal_emit_by_name (
+					GTK_OBJECT (fl), "scroll_vertical", GTK_SCROLL_STEP_BACKWARD, 0.0, NULL);
 				return FALSE;
 
 			case GDK_Down:
 			case GDK_KP_Down:
 				fl->priv->shift_down = TRUE;
-				event->state -= GDK_SHIFT_MASK;
-				event->keyval = GDK_Down;
+				gtk_signal_emit_by_name (
+					GTK_OBJECT (fl), "scroll_vertical", GTK_SCROLL_STEP_FORWARD, 0.0, NULL);
 				return FALSE;
 			
 			case GDK_Home:
 			case GDK_KP_Home:
 				fl->priv->shift_down = TRUE;
-				GTK_CLIST (fl)->focus_row = 0;
-				on_scroll_vertical (GTK_CLIST (fl), GTK_SCROLL_JUMP, 0, fl);
-				focus_file_at_row (fl, 0);
-				gtk_clist_moveto (GTK_CLIST (fl), 0, 0, 0, 0);
-				return FALSE;
+				gtk_signal_emit_by_name (
+					GTK_OBJECT (fl), "scroll_vertical", GTK_SCROLL_JUMP, 0.0);
+				return TRUE;
 				
 			case GDK_End:
 			case GDK_KP_End:
 				fl->priv->shift_down = TRUE;
-				GTK_CLIST (fl)->focus_row = num_files-1;
-				on_scroll_vertical (GTK_CLIST (fl), GTK_SCROLL_JUMP, 0, fl);
-				focus_file_at_row (fl, num_files-1);
-				gtk_clist_moveto (GTK_CLIST (fl), num_files-1, 0, 1, 0);
-				return FALSE;
+				gtk_signal_emit_by_name (
+					GTK_OBJECT (fl), "scroll_vertical", GTK_SCROLL_JUMP, 1.0);
+				return TRUE;
 		}
 	}
 	else if (state_is_ctrl (event->state)) {
@@ -2386,18 +2537,10 @@ gnome_cmd_file_list_keypressed (GnomeCmdFileList *fl,
 				
 			case GDK_Insert:
 			case GDK_KP_Insert:
-			{
-				/* toggle the current file and then emulate the
-				   down arrow being pressed */
-				static GdkEventKey down_event;
-				gboolean ret;
 				gnome_cmd_file_list_toggle (fl);
-				memcpy (&down_event, event, sizeof (GdkEventKey));
-				down_event.keyval = GDK_Down;
 				gtk_signal_emit_by_name (
-					GTK_OBJECT (fl), "key-press-event", &down_event, &ret, NULL);
+					GTK_OBJECT (fl), "scroll_vertical", GTK_SCROLL_STEP_FORWARD, 0.0, NULL);
 				return TRUE;
-			}
 
 			case GDK_KP_Page_Up:
 				event->keyval = GDK_Page_Up;
@@ -2417,16 +2560,14 @@ gnome_cmd_file_list_keypressed (GnomeCmdFileList *fl,
 			
 			case GDK_Home:
 			case GDK_KP_Home:
-				focus_file_at_row (fl, 0);
-				on_scroll_vertical (GTK_CLIST (fl), GTK_SCROLL_JUMP, 0, fl);
-				gtk_clist_moveto (GTK_CLIST (fl), 0, 0, 0, 0);
+				gtk_signal_emit_by_name (
+					GTK_OBJECT (fl), "scroll_vertical", GTK_SCROLL_JUMP, 0.0);
 				return TRUE;
 
 			case GDK_End:
 			case GDK_KP_End:
-				focus_file_at_row (fl, num_files-1);
-				on_scroll_vertical (GTK_CLIST (fl), GTK_SCROLL_JUMP, 0, fl);
-				gtk_clist_moveto (GTK_CLIST (fl), num_files-1, 0, 1, 0);
+				gtk_signal_emit_by_name (
+					GTK_OBJECT (fl), "scroll_vertical", GTK_SCROLL_JUMP, 1.0);
 				return TRUE;
 
 			case GDK_Delete:
@@ -2514,5 +2655,3 @@ gnome_vfs_list_sort (GList *list,
 		 compare_func,
 		 data);
 }
-
-
