@@ -17,23 +17,24 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */ 
 #include <config.h>
-#include <pthread.h>
+#include <libgnomevfs/gnome-vfs-mime-monitor.h>
 #include "gnome-cmd-includes.h"
-#include "widget-factory.h"
+#include "gnome-cmd-dir.h"
 #include "gnome-cmd-file-props-dialog.h"
-#include "gnome-cmd-file-funcs.h"
-#include "gnome-cmd-dir-funcs.h"
 #include "gnome-cmd-chown-component.h"
 #include "gnome-cmd-chmod-component.h"
 #include "gnome-cmd-data.h"
 #include "utils.h"
-#include "support.h"
 #include "imageloader.h"
 
 
 typedef struct {
 	GtkWidget *dialog;
 	GnomeCmdFile *finfo;
+	GThread *thread;
+	GMutex *mutex;
+	gboolean count_done;
+	gchar *msg;
 	
 	/* Properties tab stuff */
 	gboolean stop;
@@ -41,13 +42,12 @@ typedef struct {
 	GnomeVFSURI *uri;
 	GtkWidget *filename_entry;
 	GtkWidget *size_label;
-	GtkWidget *calc_button;
-	pthread_t thread;
+	GtkWidget *app_label;
 	
 	/* Permissions tab stuff */
 	GtkWidget *chown_component;
 	GtkWidget *chmod_component;
-	
+
 } GnomeCmdFilePropsDialogPrivate;
 
 
@@ -66,7 +66,7 @@ get_size_disp_string (GnomeVFSFileSize size)
 
 
 static void
-_calc_tree_size (GnomeCmdFilePropsDialogPrivate *data, GnomeVFSURI *uri)
+calc_tree_size_r (GnomeCmdFilePropsDialogPrivate *data, GnomeVFSURI *uri)
 {
 	GnomeVFSFileInfoOptions infoOpts = 0;
 	GnomeVFSResult result;
@@ -74,14 +74,14 @@ _calc_tree_size (GnomeCmdFilePropsDialogPrivate *data, GnomeVFSURI *uri)
 	gchar *uri_str;
 
 	if (data->stop)
-		pthread_exit (NULL);
+		g_thread_exit (NULL);
 	
 	if (!uri) return;
 	uri_str = gnome_vfs_uri_to_string (uri, 0);
 	if (!uri_str) return;
 
 	result = gnome_vfs_directory_list_load (
-		&list, uri_str, infoOpts, NULL);
+		&list, uri_str, infoOpts);
 
 	if (result != GNOME_VFS_OK) return;	
 	if (!list) return;
@@ -93,7 +93,7 @@ _calc_tree_size (GnomeCmdFilePropsDialogPrivate *data, GnomeVFSURI *uri)
 		if (strcmp (info->name, ".") != 0 && strcmp (info->name, "..") != 0) {		
 			if (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
 				GnomeVFSURI *new_uri = gnome_vfs_uri_append_file_name (uri, info->name);
-				_calc_tree_size (data, new_uri);
+				calc_tree_size_r (data, new_uri);
 				gnome_vfs_uri_unref (new_uri);
 			}
 			else
@@ -114,21 +114,21 @@ _calc_tree_size (GnomeCmdFilePropsDialogPrivate *data, GnomeVFSURI *uri)
 	g_free (uri_str);
 
 	if (data->stop)
-		pthread_exit (NULL);
-	
-	if (data->size_label) {
-		gdk_threads_enter ();
-		gtk_label_set_text (GTK_LABEL (data->size_label),
-							get_size_disp_string (data->size));
-		gdk_threads_leave ();
-	}
+		g_thread_exit (NULL);
+
+	g_mutex_lock (data->mutex);
+	if (data->msg) g_free (data->msg);
+	data->msg = g_strdup (get_size_disp_string (data->size));
+	g_mutex_unlock (data->mutex);
 }
 
 
 static void
-thread_func (GnomeCmdFilePropsDialogPrivate *data)
+calc_tree_size_func (GnomeCmdFilePropsDialogPrivate *data)
 {
-	_calc_tree_size (data, data->uri);
+	calc_tree_size_r (data, data->uri);
+
+	data->count_done = TRUE;
 }
 
 
@@ -138,9 +138,10 @@ thread_func (GnomeCmdFilePropsDialogPrivate *data)
 static gboolean
 join_thread_func (GnomeCmdFilePropsDialogPrivate *data)
 {
-	pthread_join (data->thread, NULL);
+	if (data->thread)
+		g_thread_join (data->thread);
+	
 	gnome_cmd_file_unref (data->finfo);
-	gnome_cmd_dir_unref (data->finfo->dir);
 	g_free (data);
 	return FALSE;
 }
@@ -150,23 +151,80 @@ static void
 on_dialog_destroy (GtkDialog *dialog, GnomeCmdFilePropsDialogPrivate *data)
 {
 	data->stop = TRUE;
+
+	gtk_timeout_add (1, (GtkFunction)join_thread_func, data);
+}
+
+
+static gboolean
+update_count_status (GnomeCmdFilePropsDialogPrivate *data)
+{
+	g_mutex_lock (data->mutex);
+	if (data->size_label) {
+		gtk_label_set_text (
+			GTK_LABEL (data->size_label), data->msg);
+	}	
+	g_mutex_unlock (data->mutex);
+
+	if (data->count_done) {
+		
+		
+		/* Returning FALSE here stops the timeout callbacks */
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+
+static void
+do_calc_tree_size (GnomeCmdFilePropsDialogPrivate *data)
+{
+	g_return_if_fail (data != NULL);
+
+	data->stop = FALSE;
+	data->size = 0;
+	data->count_done = FALSE;
+
+	data->thread = g_thread_create ((PthreadFunc)calc_tree_size_func, data, TRUE, NULL);
+
+	gtk_timeout_add (gnome_cmd_data_get_gui_update_rate (),
+					 (GtkFunction)update_count_status,
+					 data);
+}
+
+
+static void
+on_change_default_app (GtkButton *btn, GnomeCmdFilePropsDialogPrivate *data)
+{
+	gchar *appname;
+	GnomeVFSMimeApplication *vfs_app;
 	
-	if (!data->thread) {
-		gnome_cmd_file_unref (data->finfo);
-		gnome_cmd_dir_unref (data->finfo->dir);
-		g_free (data);
+	edit_mimetypes (data->finfo->info->mime_type, TRUE);
+	
+	vfs_app = gnome_vfs_mime_get_default_application (data->finfo->info->mime_type);
+	if (vfs_app) {
+		appname = vfs_app->name;
 	}
 	else
-		gtk_timeout_add (1, (GtkFunction)join_thread_func, data);
+		appname = g_strdup (_("No default application registered"));
+	
+	gtk_label_set_text (GTK_LABEL (data->app_label), appname);
+
+	if (vfs_app)
+		gnome_vfs_mime_application_free (vfs_app);
 }
 
 
+//----------------------------------------------------------------------------------
+
+
 static void
 on_dialog_ok (GtkButton *btn, GnomeCmdFilePropsDialogPrivate *data)
 {
 	GnomeVFSResult result = GNOME_VFS_OK;
 	
-	gchar *filename = gtk_entry_get_text (GTK_ENTRY (data->filename_entry));
+	const gchar *filename = gtk_entry_get_text (GTK_ENTRY (data->filename_entry));
 	if (strcmp (filename, gnome_cmd_file_get_name (data->finfo)) != 0) {
 		result = gnome_cmd_file_rename (data->finfo, filename);
 	}
@@ -212,19 +270,6 @@ on_dialog_cancel (GtkButton *btn, GnomeCmdFilePropsDialogPrivate *data)
 }
 
 
-static void
-on_calc_tree_size (GtkButton *button, GnomeCmdFilePropsDialogPrivate *data)
-{
-	g_return_if_fail (data != NULL);
-
-	data->stop = FALSE;
-	data->size = 0;
-
-	gtk_widget_set_sensitive (data->calc_button, FALSE);
-	pthread_create (&data->thread, NULL, (PthreadFunc)thread_func, data);
-}
-
-
 static void
 add_sep (GtkWidget *table, gint y)
 {
@@ -244,22 +289,28 @@ create_properties_tab (GnomeCmdFilePropsDialogPrivate *data)
 	GtkWidget *table;
 	GtkWidget *label;
 	GtkWidget *space_frame;
-
+	GtkWidget *hbox;
+	GtkWidget *btn;
+	gchar *fname;
+	GnomeVFSMimeApplication *vfs_app;
+	
 	space_frame = create_space_frame (dialog, 6);
 	
 	table = create_table (dialog, 6, 3);
 	gtk_container_add (GTK_CONTAINER (space_frame), table);
 	
-	label = create_label (dialog, _("Filename:"));
+	label = create_bold_label (dialog, _("Filename:"));
 	table_add (table, label, 0, y, GTK_FILL);
 
+	fname = get_utf8 (gnome_cmd_file_get_name (data->finfo));
 	data->filename_entry = create_entry (
-		dialog, "filename_entry", gnome_cmd_file_get_name (data->finfo));
+		dialog, "filename_entry", fname);
+	g_free (fname);
 	table_add (table, data->filename_entry, 1, y++, GTK_FILL|GTK_EXPAND);
 	gtk_editable_set_position (GTK_EDITABLE (data->filename_entry), 0);
 
 	if (data->finfo->info->symlink_name) {
-		label = create_label (dialog, _("Symlink target:"));
+		label = create_bold_label (dialog, _("Symlink target:"));
 		table_add (table, label, 0, y, GTK_FILL);
 	
 		label = create_label (dialog, data->finfo->info->symlink_name);
@@ -268,31 +319,52 @@ create_properties_tab (GnomeCmdFilePropsDialogPrivate *data)
 
 	add_sep (table, y++);
 	
-	label = create_label (dialog, _("Type:"));
+	label = create_bold_label (dialog, _("Type:"));
 	table_add (table, label, 0, y, GTK_FILL);
 	
 	label = create_label (dialog, gnome_cmd_file_get_mime_type_desc (data->finfo));
 	table_add (table, label, 1, y++, GTK_FILL);
 
 
-	label = create_label (dialog, _("MIME Type:"));
+	label = create_bold_label (dialog, _("MIME Type:"));
 	table_add (table, label, 0, y, GTK_FILL);
 
 	label = create_label (dialog, gnome_cmd_file_get_mime_type (data->finfo));
 	table_add (table, label, 1, y++, GTK_FILL);
+
+
+	if (data->finfo->info->type != GNOME_VFS_FILE_TYPE_DIRECTORY) {
+		label = create_bold_label (dialog, _("Opens with:"));
+		table_add (table, label, 0, y, GTK_FILL);
 	
+		vfs_app = gnome_vfs_mime_get_default_application (data->finfo->info->mime_type);
+		if (vfs_app) {
+			data->app_label = label = create_label (dialog, vfs_app->name);
+			gnome_vfs_mime_application_free (vfs_app);
+		}
+		else
+			label = create_label (dialog, _("No default application registered"));
+		btn = create_button_with_data (
+			dialog, _("_Change"), GTK_SIGNAL_FUNC (on_change_default_app), data);
+		hbox = create_hbox (dialog, FALSE, 6);
+		gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
+		label = create_label (dialog, " ");
+		gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
+		gtk_box_pack_start (GTK_BOX (hbox), btn, FALSE, TRUE, 0);
+		table_add (table, hbox, 1, y++, GTK_FILL);
+	}
 
 	add_sep (table, y++);
 	
 	
-	label = create_label (dialog, _("Accessed:"));
+	label = create_bold_label (dialog, _("Accessed:"));
 	table_add (table, label, 0, y, GTK_FILL);
 
 	label = create_label (dialog, gnome_cmd_file_get_adate (data->finfo, TRUE));
 	table_add (table, label, 1, y++, GTK_FILL);
 
 
-	label = create_label (dialog, _("Modified:"));
+	label = create_bold_label (dialog, _("Modified:"));
 	table_add (table, label, 0, y, GTK_FILL);
 
 	label = create_label (dialog, gnome_cmd_file_get_mdate (data->finfo, TRUE));
@@ -302,32 +374,16 @@ create_properties_tab (GnomeCmdFilePropsDialogPrivate *data)
 	add_sep (table, y++);
 	
 	
-	label = create_label (dialog, _("Size:"));
+	label = create_bold_label (dialog, _("Size:"));
 	table_add (table, label, 0, y, GTK_FILL);
 
 	label = create_label (dialog, get_size_disp_string (data->finfo->info->size));
-	if (data->finfo->info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
-		GtkWidget *hbox;
-		GtkWidget *button;
-		
-		hbox = create_hbox (dialog, FALSE, 12);
-		gtk_container_set_border_width (GTK_CONTAINER (hbox), 0);
-		table_add (table, hbox, 1, y++, GTK_FILL);
-		
-		gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-		
-		button = create_button_with_data (dialog, _("_Count Total Size"),
-													 on_calc_tree_size, data);
-		gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
-		data->calc_button = button;
-	}
-	else {
-		table_add (table, label, 1, y++, GTK_FILL);
-	}
+	table_add (table, label, 1, y++, GTK_FILL);
+	if (data->finfo->info->type == GNOME_VFS_FILE_TYPE_DIRECTORY)
+		do_calc_tree_size (data);
+	
 	data->size_label = label;
 
-
-
 	return space_frame;
 }
 
@@ -380,7 +436,6 @@ gnome_cmd_file_props_dialog_create (GnomeCmdFile *finfo)
 {
 	GtkWidget *dialog;
 	GtkWidget *notebook;
-	GtkWidget *space_frame;
 	GnomeCmdFilePropsDialogPrivate *data;
 	
 	g_return_val_if_fail (finfo != NULL, NULL);
@@ -392,25 +447,22 @@ gnome_cmd_file_props_dialog_create (GnomeCmdFile *finfo)
 	data = g_new (GnomeCmdFilePropsDialogPrivate, 1);
 	data->thread = 0;
 
-	dialog = create_ok_cancel_dialog (_("File Properties"),
-									  on_dialog_ok, on_dialog_cancel, data);
+	dialog = gnome_cmd_dialog_new (_("File Properties"));
 	gtk_signal_connect (GTK_OBJECT (dialog), "destroy", (GtkSignalFunc)on_dialog_destroy, data);
 
 	data->dialog = GTK_WIDGET (dialog);
 	data->finfo = finfo;
 	data->uri = gnome_cmd_file_get_uri (finfo);
+	data->mutex = g_mutex_new ();
+	data->msg = NULL;
 	gnome_cmd_file_ref (finfo);
-	gnome_cmd_dir_ref (finfo->dir);
-
-	space_frame = create_space_frame (dialog, 0);
-	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), space_frame, TRUE, TRUE, 0);
 
 	notebook = gtk_notebook_new ();
 	gtk_widget_ref (notebook);
 	gtk_object_set_data_full (GTK_OBJECT (dialog), "notebook", notebook,
 							  (GtkDestroyNotify) gtk_widget_unref);
 	gtk_widget_show (notebook);
-	gtk_container_add (GTK_CONTAINER (space_frame), notebook);
+	gnome_cmd_dialog_add_category (GNOME_CMD_DIALOG (dialog), notebook);
 
 	gtk_container_add (GTK_CONTAINER (notebook),
 					   create_properties_tab (data));
@@ -425,6 +477,13 @@ gnome_cmd_file_props_dialog_create (GnomeCmdFile *finfo)
 		GTK_NOTEBOOK (notebook),
 		gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 1),
 		gtk_label_new (_("Permissions")));
+
+	gnome_cmd_dialog_add_button (
+		GNOME_CMD_DIALOG (dialog), GNOME_STOCK_BUTTON_CANCEL,
+		GTK_SIGNAL_FUNC (on_dialog_cancel), data);
+	gnome_cmd_dialog_add_button (
+		GNOME_CMD_DIALOG (dialog), GNOME_STOCK_BUTTON_OK,
+		GTK_SIGNAL_FUNC (on_dialog_ok), data);
 	
 	return dialog;
 }
