@@ -19,10 +19,7 @@
 #ifndef __GNOME_CMD_MAKE_COPY_DIALOG_H__
 #define __GNOME_CMD_MAKE_COPY_DIALOG_H__
 
-#include "gnome-cmd-file-types.h"
-#include "gnome-cmd-string-dialog.h"
-
-BEGIN_GNOME_DECLS
+#include "gnome-cmd-file.h"
 
 #define GNOME_CMD_MAKE_COPY_DIALOG(obj) \
 	GTK_CHECK_CAST (obj, gnome_cmd_make_copy_dialog_get_type (), GnomeCmdMakeCopyDialog)
@@ -52,7 +49,7 @@ struct _GnomeCmdMakeCopyDialogClass
 
 
 GtkWidget*
-gnome_cmd_make_copy_dialog_new (GnomeCmdFile *finfo);
+gnome_cmd_make_copy_dialog_new (GnomeCmdFile *finfo, GnomeCmdDir *dir);
 
 GtkType
 gnome_cmd_make_copy_dialog_get_type (void);
