@@ -19,8 +19,8 @@
 #include <config.h>
 #include "gnome-cmd-includes.h"
 #include "gnome-cmd-mkdir-dialog.h"
-#include "gnome-cmd-dir-funcs.h"
-#include "gnome-cmd-main-win-funcs.h"
+#include "gnome-cmd-dir.h"
+#include "gnome-cmd-main-win.h"
 #include "utils.h"
 
 
@@ -49,7 +49,7 @@ on_ok (GnomeCmdStringDialog *string_dialog, const gchar **values, GnomeCmdMkdirD
 		return FALSE;
 	}
 
-	uri = gnome_cmd_dir_get_file_uri (dialog->priv->dir, filename);
+	uri = gnome_cmd_dir_get_child_uri (dialog->priv->dir, filename);
 
 	result = gnome_vfs_make_directory_for_uri (
 		uri,
@@ -147,7 +147,7 @@ gnome_cmd_mkdir_dialog_new (GnomeCmdDir *dir)
 		labels,
 		1,
 		(GnomeCmdStringDialogCallback)on_ok,
-		(GnomeStringCallback)on_cancel,
+		(GtkSignalFunc)on_cancel,
 		dialog);
 	
 	return GTK_WIDGET (dialog);
