@@ -1,6 +1,6 @@
 /*
     GNOME Commander - A GNOME based file manager 
-    Copyright (C) 2001-2002 Marcus Bjurman
+    Copyright (C) 2001-2003 Marcus Bjurman
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,14 +19,11 @@
 #ifndef __GNOME_CMD_OPTIONS_DIALOG_H__
 #define __GNOME_CMD_OPTIONS_DIALOG_H__
 
-
-BEGIN_GNOME_DECLS
-
 #define GNOME_CMD_OPTIONS_DIALOG(obj) \
 	GTK_CHECK_CAST (obj, gnome_cmd_options_dialog_get_type (), GnomeCmdOptionsDialog)
 #define GNOME_CMD_OPTIONS_DIALOG_CLASS(klass) \
 	GTK_CHECK_CLASS_CAST (klass, gnome_cmd_options_dialog_get_type (), GnomeCmdOptionsDialogClass)
-#define GNOME_IS_CMD_OPTIONS_DIALOG(obj) \
+#define GNOME_CMD_IS_OPTIONS_DIALOG(obj) \
 	GTK_CHECK_TYPE (obj, gnome_cmd_options_dialog_get_type ())
 
 
@@ -37,20 +34,18 @@ typedef struct _GnomeCmdOptionsDialogClass GnomeCmdOptionsDialogClass;
 
 typedef enum
 {
-	GNOME_CMD_OPTIONS_DIALOG_TAB_HIDE,
+	GNOME_CMD_OPTIONS_DIALOG_TAB_GENERAL,
+	GNOME_CMD_OPTIONS_DIALOG_TAB_FORMAT,
 	GNOME_CMD_OPTIONS_DIALOG_TAB_LAYOUT,
-	GNOME_CMD_OPTIONS_DIALOG_TAB_SORT,
-	GNOME_CMD_OPTIONS_DIALOG_TAB_DATES,
-//	GNOME_CMD_OPTIONS_DIALOG_TAB_CVS,
-//	GNOME_CMD_OPTIONS_DIALOG_TAB_FTP,
-	GNOME_CMD_OPTIONS_DIALOG_TAB_EX_PROGS,
-	GNOME_CMD_OPTIONS_DIALOG_TAB_OPTIONS
+	GNOME_CMD_OPTIONS_DIALOG_TAB_FILTERS,
+	GNOME_CMD_OPTIONS_DIALOG_TAB_PROGRAMS,
+	GNOME_CMD_OPTIONS_DIALOG_TAB_DEVICES
 } GnomeCmdOptionsDialogTab;
 
 
 struct _GnomeCmdOptionsDialog
 {
-	GnomeDialog parent;
+	GnomeCmdDialog parent;
 
 	GtkWidget *notebook;
 	
@@ -60,7 +55,7 @@ struct _GnomeCmdOptionsDialog
 
 struct _GnomeCmdOptionsDialogClass
 {
-	GnomeDialogClass parent_class;
+	GnomeCmdDialogClass parent_class;
 };
 
 
@@ -69,7 +64,7 @@ GtkType
 gnome_cmd_options_dialog_get_type            (void);
 
 GtkWidget*
-gnome_cmd_options_dialog_new                 (GnomeCmdMainWin *mw);
+gnome_cmd_options_dialog_new                 (void);
 
 void
 gnome_cmd_options_dialog_set_tab             (GnomeCmdOptionsDialog *dialog,
