@@ -20,14 +20,12 @@
 #include "gnome-cmd-includes.h"
 #include "gnome-cmd-prepare-copy-dialog.h"
 #include "gnome-cmd-prepare-xfer-dialog.h"
-#include "gnome-cmd-dir-funcs.h"
-#include "dir_pool.h"
+#include "gnome-cmd-dir.h"
 #include "gnome-cmd-xfer.h"
-#include "gnome-cmd-connection-funcs.h"
-#include "gnome-cmd-dir-funcs.h"
-#include "gnome-cmd-file-funcs.h"
-#include "gnome-cmd-file-selector-funcs.h"
-#include "gnome-cmd-main-win-funcs.h"
+#include "gnome-cmd-con.h"
+#include "gnome-cmd-file-selector.h"
+#include "gnome-cmd-main-win.h"
+#include "utils.h"
 
 typedef struct
 {
@@ -62,20 +60,21 @@ on_ok (GtkButton *button, gpointer user_data)
 
 
 void
-gnome_cmd_prepare_copy_dialog_show (GnomeCmdFileList *from,
+gnome_cmd_prepare_copy_dialog_show (GnomeCmdFileSelector *from,
 									GnomeCmdFileSelector *to)
 {
 	GSList *group = NULL;
 	PrepareCopyData *data;
 	GnomeCmdFile *finfo;
 	gint num_files;
-	gchar *dest_dir_frame_msg;
+	gchar *dest_dir_frame_msg, *text;
 	GList *tmp;
+	GtkWidget *label;
 
-	g_return_if_fail (GNOME_CMD_IS_FILE_LIST (from));
+	g_return_if_fail (GNOME_CMD_IS_FILE_SELECTOR (from));
 	g_return_if_fail (GNOME_CMD_IS_FILE_SELECTOR (to));
 
-	tmp = gnome_cmd_file_list_get_selected_files (from);
+	tmp = gnome_cmd_file_list_get_selected_files (from->list);
 	if (!tmp) {
 		g_list_free (tmp);
 		return;
@@ -134,23 +133,32 @@ gnome_cmd_prepare_copy_dialog_show (GnomeCmdFileList *from,
 	/*
 	 * Customize prepare xfer widgets 
 	 */
-	gtk_frame_set_label (GTK_FRAME (data->dialog->left_vbox_frame),
-						 _("Overwrite Files"));
+	text = get_bold_text (_("Overwrite Files"));
+	label = gtk_object_get_data (GTK_OBJECT (data->dialog->left_vbox_frame), "label");
+	gtk_label_set_markup (GTK_LABEL (label), text);
+	g_free (text);
 	
-	gtk_frame_set_label (GTK_FRAME (data->dialog->right_vbox_frame),
-						 _("Options"));
+	text = get_bold_text (_("Options"));
+	label = gtk_object_get_data (GTK_OBJECT (data->dialog->right_vbox_frame), "label");
+	gtk_label_set_markup (GTK_LABEL (label), text);
+	g_free (text);
 
 	g_return_if_fail (data->dialog->src_files != NULL);
 	num_files = g_list_length (data->dialog->src_files);
 	finfo = (GnomeCmdFile*)data->dialog->src_files->data;
 	
-	if (num_files == 1)
-		dest_dir_frame_msg = g_strdup_printf (_("Copy \"%s\" to"), finfo->info->name);
+	if (num_files == 1) {
+		gchar *fname = get_utf8 (finfo->info->name);
+		dest_dir_frame_msg = g_strdup_printf (_("Copy \"%s\" to"), fname);
+		g_free (fname);
+	}
 	else
 		dest_dir_frame_msg = g_strdup_printf (ngettext("copy %d file to","copy %d files to",num_files), num_files);
-	
-	gtk_frame_set_label (GTK_FRAME (data->dialog->dest_dir_frame),
-						 dest_dir_frame_msg);
+
+	text = get_bold_text (dest_dir_frame_msg);
+	label = gtk_object_get_data (GTK_OBJECT (data->dialog->dest_dir_frame), "label");
+	gtk_label_set_markup (GTK_LABEL (label), text);
+	g_free (text);
 
 	g_free  (dest_dir_frame_msg);
 
@@ -160,7 +168,7 @@ gnome_cmd_prepare_copy_dialog_show (GnomeCmdFileList *from,
 	 * Connect signals
 	 */
 	gtk_signal_connect (GTK_OBJECT (data->dialog->ok_button),
-						"clicked", on_ok, data);
+						"clicked", GTK_SIGNAL_FUNC (on_ok), data);
 
 
 	/*
