@@ -20,10 +20,10 @@
 #define __GNOME_CMD_PREPARE_COPY_DIALOG_H__
 
 #include "gnome-cmd-file-list.h"
-#include "gnome-cmd-file-selector-types.h"
+#include "gnome-cmd-file-selector.h"
 
 void
-gnome_cmd_prepare_copy_dialog_show (GnomeCmdFileList *from,
+gnome_cmd_prepare_copy_dialog_show (GnomeCmdFileSelector *fs,
 									GnomeCmdFileSelector *to);
 
 #endif //__GNOME_CMD_PREPARE_COPY_DIALOG_H__
