@@ -19,11 +19,9 @@
 #ifndef __GNOME_CMD_PREPARE_XFER_DIALOG_H__
 #define __GNOME_CMD_PREPARE_XFER_DIALOG_H__
 
-#include "gnome-cmd-dir-types.h"
+#include "gnome-cmd-dir.h"
 #include "gnome-cmd-file-list.h"
-#include "gnome-cmd-file-selector-types.h"
-
-BEGIN_GNOME_DECLS
+#include "gnome-cmd-file-selector.h"
 
 #define GNOME_CMD_PREPARE_XFER_DIALOG(obj) \
 	GTK_CHECK_CAST (obj, gnome_cmd_prepare_xfer_dialog_get_type (), GnomeCmdPrepareXferDialog)
@@ -40,7 +38,7 @@ typedef struct _GnomeCmdPrepareXferDialogClass GnomeCmdPrepareXferDialogClass;
 
 struct _GnomeCmdPrepareXferDialog
 {
-	GnomeDialog parent;
+	GnomeCmdDialog parent;
 
 	GtkWidget *dest_dir_frame;
 	GtkWidget *dest_dir_entry;
@@ -55,20 +53,19 @@ struct _GnomeCmdPrepareXferDialog
 	GnomeVFSXferOverwriteMode xferOverwriteMode;
 
 	GList *src_files;
-	GnomeCmdFileList *src_fl;
-	GnomeCmdDir *dest_dir;
-	GnomeVFSURI *dest_baseuri;
+	GnomeCmdFileSelector *src_fs;
+	GnomeCmdDir *default_dest_dir;
 };
 
 
 struct _GnomeCmdPrepareXferDialogClass
 {
-	GnomeDialogClass parent_class;
+	GnomeCmdDialogClass parent_class;
 };
 
 
 GtkWidget*
-gnome_cmd_prepare_xfer_dialog_new (GnomeCmdFileList *from,
+gnome_cmd_prepare_xfer_dialog_new (GnomeCmdFileSelector *from,
 								   GnomeCmdFileSelector *to);
 
 GtkType
