@@ -18,10 +18,10 @@
 */ 
 #include <config.h>
 #include "gnome-cmd-includes.h"
-#include "widget-factory.h"
 #include "gnome-cmd-rename-dialog.h"
-#include "gnome-cmd-file-funcs.h"
-#include "gnome-cmd-main-win-funcs.h"
+#include "gnome-cmd-file.h"
+#include "gnome-cmd-main-win.h"
+#include "utils.h"
 
 
 struct _GnomeCmdRenameDialogPrivate
@@ -131,6 +131,7 @@ gnome_cmd_rename_dialog_new (GnomeCmdFile *finfo)
 	const gchar *labels[] = {""};
 	gchar *msg;
 	GtkWidget *msg_label;
+	gchar *fname;
 	GnomeCmdRenameDialog *dialog;
 
 	g_return_val_if_fail (finfo != NULL, NULL);
@@ -140,24 +141,25 @@ gnome_cmd_rename_dialog_new (GnomeCmdFile *finfo)
 	dialog->priv->finfo = finfo;
 	gnome_cmd_file_ref (finfo);
 
-	msg = g_strdup_printf (_("Rename \"%s\" to"), gnome_cmd_file_get_name (finfo));
+	fname = get_utf8 (gnome_cmd_file_get_name (finfo));
+	msg = g_strdup_printf (_("Rename \"%s\" to"), fname);
 	msg_label = create_label (GTK_WIDGET (dialog), msg);
 	g_free (msg);
-	
-	gtk_box_pack_start (GTK_BOX (GNOME_CMD_STRING_DIALOG (dialog)->parent.vbox),
-						msg_label, FALSE, TRUE, 0);
 
+	gnome_cmd_dialog_add_category (GNOME_CMD_DIALOG (dialog), msg_label);
+	
 	gnome_cmd_string_dialog_setup_with_cancel (
 		GNOME_CMD_STRING_DIALOG (dialog),
 		_("Rename File"),
 		labels,
 		1,
 		(GnomeCmdStringDialogCallback)on_ok,
-		(GnomeStringCallback)on_cancel,
+		GTK_SIGNAL_FUNC (on_cancel),
 		dialog);
 
 	gnome_cmd_string_dialog_set_value (
-		GNOME_CMD_STRING_DIALOG (dialog), 0, finfo->info->name);
+		GNOME_CMD_STRING_DIALOG (dialog), 0, fname);
+	g_free (fname);
 	
 	return GTK_WIDGET (dialog);
 }
