@@ -19,11 +19,9 @@
 #ifndef __GNOME_CMD_SEARCH_DIALOG_H__
 #define __GNOME_CMD_SEARCH_DIALOG_H__
 
-#include "gnome-cmd-dir-types.h"
+#include "gnome-cmd-dir.h"
 
 
-BEGIN_GNOME_DECLS
-
 #define GNOME_CMD_SEARCH_DIALOG(obj) \
 	GTK_CHECK_CAST (obj, gnome_cmd_search_dialog_get_type (), GnomeCmdSearchDialog)
 #define GNOME_CMD_SEARCH_DIALOG_CLASS(klass) \
@@ -40,7 +38,7 @@ typedef struct _GnomeCmdSearchDialogClass GnomeCmdSearchDialogClass;
 
 struct _GnomeCmdSearchDialog
 {
-	GnomeDialog parent;
+	GnomeCmdDialog parent;
 
 	GnomeCmdSearchDialogPrivate *priv;
 };
@@ -48,7 +46,7 @@ struct _GnomeCmdSearchDialog
 
 struct _GnomeCmdSearchDialogClass
 {
-	GnomeDialogClass parent_class;
+	GnomeCmdDialogClass parent_class;
 };
 
 
@@ -56,7 +54,7 @@ GtkType
 gnome_cmd_search_dialog_get_type            (void);
 
 GtkWidget*
-gnome_cmd_search_dialog_new                 (GnomeVFSURI *baseuri, GnomeCmdDir *default_dir);
+gnome_cmd_search_dialog_new                 (GnomeCmdDir *default_dir);
 
 
 #endif //__GNOME_CMD_SEARCH_DIALOG_H__
