@@ -21,49 +21,40 @@
 #include "gnome-cmd-data.h"
 #include "gnome-cmd-types.h"
 
-GtkRcStyle *main_rc_style = NULL;
 GtkStyle *list_style = NULL;
 GtkStyle *sel_list_style = NULL;
 
 
-static GtkRcStyle *
-create_main_rc_style ()
-{
-	GtkRcStyle *style;
-
-	style = gtk_rc_style_new ();
-	style->font_name = g_strdup (gnome_cmd_data_get_misc_font ());
-	
-	return style;
-}
-
-
 static GtkStyle *
 create_list_style ()
 {
 	GtkStyle *style;
-	GdkFont *font = NULL;
-	GnomeCmdLayoutColors *cols = gnome_cmd_data_get_layout_colors ();
+	GnomeCmdColorTheme *cols = gnome_cmd_data_get_current_color_theme ();
 	const gchar *font_name = gnome_cmd_data_get_list_font ();
 
-	if (strcmp (font_name, "default") != 0) {
-		font = gdk_font_load (font_name);		
-	}
-
 	style = gtk_style_new ();
 
-	if (font) {
-		gdk_font_unref (style->font);
-		gdk_font_ref (font);
-		style->font = font;
+	if (strcmp (font_name, "default") != 0) {
+		PangoFontDescription *pfont;
+		pfont = pango_font_description_from_string (font_name);
+
+		if (pfont
+			&& pango_font_description_get_size (pfont) != 0) {
+			if (style->font_desc)
+				pango_font_description_free (style->font_desc);
+			style->font_desc = pfont;
+		}
 	}
 
-	style->fg[GTK_STATE_SELECTED] = *cols->curs_fg;
-	style->fg[GTK_STATE_NORMAL] = *cols->norm_fg;
+	if (!cols->respect_theme) {
+		style->fg[GTK_STATE_SELECTED] = *cols->curs_fg;
+		style->text[GTK_STATE_NORMAL] = *cols->norm_fg;
+		style->fg[GTK_STATE_NORMAL] = *cols->norm_fg;
 
-	style->bg[GTK_STATE_SELECTED] = *cols->curs_bg;
-	style->base[GTK_STATE_NORMAL] = *cols->norm_bg;
-	style->base[GTK_STATE_ACTIVE] = *cols->norm_bg;
+		style->bg[GTK_STATE_SELECTED] = *cols->curs_bg;
+		style->base[GTK_STATE_NORMAL] = *cols->norm_bg;
+		style->base[GTK_STATE_ACTIVE] = *cols->norm_bg;
+	}
 	
 	return style;
 }
@@ -73,29 +64,33 @@ static GtkStyle *
 create_sel_list_style ()
 {
 	GtkStyle *style;
-	GdkFont *font = NULL;
-	GnomeCmdLayoutColors *cols = gnome_cmd_data_get_layout_colors ();
+	GnomeCmdColorTheme *cols = gnome_cmd_data_get_current_color_theme ();
 	const gchar *font_name = gnome_cmd_data_get_list_font ();
 
-	if (strcmp (font_name, "default") != 0) {
-		font = gdk_font_load (font_name);		
-	}
-
 	style = gtk_style_new ();
 
-	if (font) {
-		gdk_font_unref (style->font);
-		gdk_font_ref (font);
-		style->font = font;
+	if (strcmp (font_name, "default") != 0) {
+		PangoFontDescription *pfont;
+		pfont = pango_font_description_from_string (font_name);
+
+		if (pfont &&
+			pango_font_description_get_size (pfont) != 0) {
+			if (style->font_desc)
+				pango_font_description_free (style->font_desc);
+			style->font_desc = pfont;
+		}
 	}
+	
+	if (!cols->respect_theme) {
+		style->fg[GTK_STATE_SELECTED] = *cols->sel_fg;
+		style->fg[GTK_STATE_NORMAL] = *cols->sel_fg;
+		style->text[GTK_STATE_NORMAL] = *cols->sel_fg;
 
-	style->fg[GTK_STATE_SELECTED] = *cols->sel_fg;
-	style->fg[GTK_STATE_NORMAL] = *cols->sel_fg;
-
-	style->bg[GTK_STATE_SELECTED] = *cols->curs_bg;
-	style->bg[GTK_STATE_NORMAL] = *cols->sel_bg;
-	style->base[GTK_STATE_NORMAL] = *cols->norm_bg;
-	style->base[GTK_STATE_ACTIVE] = *cols->norm_bg;
+		style->bg[GTK_STATE_SELECTED] = *cols->curs_bg;
+		style->bg[GTK_STATE_NORMAL] = *cols->sel_bg;
+		style->base[GTK_STATE_NORMAL] = *cols->norm_bg;
+		style->base[GTK_STATE_ACTIVE] = *cols->norm_bg;
+	}
 	
 	return style;
 }
@@ -103,11 +98,9 @@ create_sel_list_style ()
 
 void gnome_cmd_style_create (void)
 {
-	if (main_rc_style) gtk_rc_style_unref (main_rc_style);
 	if (list_style) gtk_style_unref (list_style);
 	if (sel_list_style) gtk_style_unref (sel_list_style);
 	
-	main_rc_style = create_main_rc_style ();	
 	list_style = create_list_style ();
 	sel_list_style = create_sel_list_style ();
 }
