@@ -20,7 +20,6 @@
 #include "gnome-cmd-includes.h"
 #include "gnome-cmd-xfer-progress-win.h"
 #include "gnome-cmd-data.h"
-#include "widget-factory.h"
 #include "utils.h"
 
 static GtkWindowClass *parent_class = NULL;
@@ -106,7 +105,7 @@ init (GnomeCmdXferProgressWin *win)
 	bbox = create_hbuttonbox (w);
 	gtk_container_add (GTK_CONTAINER (vbox), bbox);
 
-	button = create_stock_button (w, GNOME_STOCK_BUTTON_CANCEL, on_cancel);
+	button = create_stock_button (w, GNOME_STOCK_BUTTON_CANCEL, GTK_SIGNAL_FUNC (on_cancel));
 	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
 	gtk_container_add (GTK_CONTAINER (bbox), button);
 }
