@@ -38,6 +38,7 @@ typedef enum {
 	PIXMAP_SERVER_SMALL,
 	PIXMAP_LOGO,
 	PIXMAP_EXEC,
+	PIXMAP_EXEC_WHEEL,
 	PIXMAP_MKDIR,
 	PIXMAP_LOCK,
 	PIXMAP_HOME,
@@ -49,7 +50,12 @@ typedef enum {
 	PIXMAP_OVERLAY_UMOUNT,
 	PIXMAP_PARENT_DIR,
 	PIXMAP_ROOT_DIR,
+	PIXMAP_FTP_CONNECT,
 	PIXMAP_FTP_DISCONNECT,
+	PIXMAP_MENU_FTP_CONNECT,
+	PIXMAP_MENU_FTP_DISCONNECT,
+	PIXMAP_SWITCH_V,
+	PIXMAP_SWITCH_H,
 	
 	NUM_PIXMAPS
 } Pixmap;
