@@ -369,7 +369,7 @@ void OWNER_free (void)
 
 user_t *OWNER_get_program_user (void)
 {
-    char *name;
+    const char *name;
     user_t *user;
 
     name = g_get_user_name ();
