@@ -19,24 +19,25 @@
 #include <config.h>
 #include "gnome-cmd-includes.h"
 #include "useractions.h"
-#include "gnome-cmd-main-win-funcs.h"
-#include "gnome-cmd-file-funcs.h"
-#include "gnome-cmd-dir-funcs.h"
-#include "gnome-cmd-file-selector-funcs.h"
+#include "gnome-cmd-main-win.h"
+#include "plugin_manager.h"
+#include "gnome-cmd-file.h"
+#include "gnome-cmd-dir.h"
+#include "gnome-cmd-file-selector.h"
 #include "gnome-cmd-file-list.h"
 #include "cap.h"
 #include "gnome-cmd-prepare-copy-dialog.h"
 #include "gnome-cmd-prepare-move-dialog.h"
 #include "gnome-cmd-data.h"
 #include "gnome-cmd-search-dialog.h"
-#include "gnome-cmd-connection-funcs.h"
+#include "gnome-cmd-con.h"
 #include "gnome-cmd-options-dialog.h"
 #include "gnome-cmd-bookmark-dialog.h"
 #include "gnome-cmd-ftp-dialog.h"
 #include "utils.h"
 
 
-static GnomeCmdFileList *get_current_fl ()
+static GnomeCmdFileList *get_active_fl ()
 {
 	GnomeCmdFileSelector *fs = gnome_cmd_main_win_get_active_fs (main_win);
 
@@ -45,22 +46,28 @@ static GnomeCmdFileList *get_current_fl ()
 }
 
 
-static GnomeCmdFileSelector *get_current_fs ()
+static GnomeCmdFileSelector *get_active_fs ()
 {
 	return gnome_cmd_main_win_get_active_fs (main_win);
 }
 
 
+static GnomeCmdFileSelector *get_inactive_fs ()
+{
+	return gnome_cmd_main_win_get_inactive_fs (main_win);
+}
+
+
 /**
  * The file returned from this function is not to be unrefed
  */
 static GnomeCmdFile *
 get_selected_file ()
 {
-	GnomeCmdFile *finfo = gnome_cmd_file_list_get_selected_file (get_current_fl ());
+	GnomeCmdFile *finfo = gnome_cmd_file_list_get_selected_file (get_active_fl ());
 	
 	if (!finfo)
-		gnome_error_dialog ("No file selected");
+		create_error_dialog ("No file selected");
 	return finfo;
 }
 
@@ -71,7 +78,7 @@ void
 file_cap_cut                        (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {
-	gnome_cmd_file_list_cap_cut (get_current_fl ());
+	gnome_cmd_file_list_cap_cut (get_active_fl ());
 }
 
 
@@ -79,7 +86,7 @@ void
 file_cap_copy                       (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {
-	gnome_cmd_file_list_cap_copy (get_current_fl ());
+	gnome_cmd_file_list_cap_copy (get_active_fl ());
 }
 
 
@@ -87,7 +94,7 @@ void
 file_cap_paste                      (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {
-	gnome_cmd_file_selector_cap_paste (get_current_fs ());
+	gnome_cmd_file_selector_cap_paste (get_active_fs ());
 }
 
 
@@ -95,15 +102,15 @@ void
 file_copy                           (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {
-	GnomeCmdFileList *src_fl;
+	GnomeCmdFileSelector *src_fs;
 	GnomeCmdFileSelector *dest_fs;
 
 	if (main_win) {
-		src_fl = get_current_fl ();
+		src_fs = gnome_cmd_main_win_get_active_fs (main_win);
 		dest_fs = gnome_cmd_main_win_get_inactive_fs (main_win);
 		
-		if (src_fl && dest_fs)
-			gnome_cmd_prepare_copy_dialog_show (src_fl, dest_fs);
+		if (src_fs && dest_fs)
+			gnome_cmd_prepare_copy_dialog_show (src_fs, dest_fs);
 	}
 }
 
@@ -112,15 +119,15 @@ void
 file_move                           (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {
-	GnomeCmdFileList *src_fl;
+	GnomeCmdFileSelector *src_fs;
 	GnomeCmdFileSelector *dest_fs;
 
 	if (main_win) {
-		src_fl = get_current_fl ();
+		src_fs = gnome_cmd_main_win_get_active_fs (main_win);
 		dest_fs = gnome_cmd_main_win_get_inactive_fs (main_win);
 		
-		if (src_fl && dest_fs)
-			gnome_cmd_prepare_move_dialog_show (src_fl, dest_fs);
+		if (src_fs && dest_fs)
+			gnome_cmd_prepare_move_dialog_show (src_fs, dest_fs);
 	}
 }
 
@@ -129,7 +136,7 @@ void
 file_delete                         (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {
-	gnome_cmd_file_list_show_delete_dialog (get_current_fl ());
+	gnome_cmd_file_list_show_delete_dialog (get_active_fl ());
 }
 
 
@@ -137,7 +144,7 @@ void
 file_view                           (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {
-	gnome_cmd_file_list_view (get_current_fl ());
+	gnome_cmd_file_list_view (get_active_fl ());
 }
 
 
@@ -145,7 +152,7 @@ void
 file_edit                           (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {
-	gnome_cmd_file_list_edit (get_current_fl ());
+	gnome_cmd_file_list_edit (get_active_fl ());
 }
 
 
@@ -153,7 +160,7 @@ void
 file_chmod                          (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {
-	gnome_cmd_file_list_show_chmod_dialog (get_current_fl ());
+	gnome_cmd_file_list_show_chmod_dialog (get_active_fl ());
 }
 
 
@@ -161,7 +168,7 @@ void
 file_chown                          (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {
-	gnome_cmd_file_list_show_chown_dialog (get_current_fl ());
+	gnome_cmd_file_list_show_chown_dialog (get_active_fl ());
 }
 
 
@@ -169,7 +176,7 @@ void
 file_mkdir                          (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {
-	gnome_cmd_file_selector_show_mkdir_dialog (get_current_fs ());
+	gnome_cmd_file_selector_show_mkdir_dialog (get_active_fs ());
 }
 
 
@@ -177,7 +184,7 @@ void
 file_create_symlink                 (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {
-	GnomeCmdFile *finfo = gnome_cmd_file_list_get_focused_file (get_current_fl ());
+	GnomeCmdFile *finfo = gnome_cmd_file_list_get_focused_file (get_active_fl ());
 	gnome_cmd_file_selector_create_symlink (
 		gnome_cmd_main_win_get_inactive_fs (main_win), finfo);
 }
@@ -187,7 +194,7 @@ void
 file_rename                         (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {
-	gnome_cmd_file_list_show_rename_dialog (get_current_fl ());
+	gnome_cmd_file_list_show_rename_dialog (get_active_fl ());
 }
 
 
@@ -195,7 +202,7 @@ void
 file_advrename                      (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {	
-	gnome_cmd_file_list_show_advrename_dialog (get_current_fl ());
+	gnome_cmd_file_list_show_advrename_dialog (get_active_fl ());
 }
 
 
@@ -203,7 +210,7 @@ void
 file_properties                     (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {
-	gnome_cmd_file_list_show_properties_dialog (get_current_fl ());
+	gnome_cmd_file_list_show_properties_dialog (get_active_fl ());
 }
 
 
@@ -216,6 +223,9 @@ file_diff                           (GtkMenuItem     *menuitem,
 	if (finfo)
 	{
 		GList *tmp;
+		gchar *cmd, *p1, *p2 = NULL;
+		GnomeCmdFile *finfo2 = NULL;
+		gboolean found = FALSE;
 		GList *all_files = gnome_cmd_file_list_get_all_files (
 			GNOME_CMD_FILE_LIST (gnome_cmd_main_win_get_inactive_fs (main_win)->list));
 		tmp = all_files;
@@ -225,55 +235,40 @@ file_diff                           (GtkMenuItem     *menuitem,
 		 */
 		while (tmp)
 		{
-			GnomeCmdFile *finfo2 = (GnomeCmdFile*)tmp->data;
+			finfo2 = (GnomeCmdFile*)tmp->data;
 
 			if (strcmp (gnome_cmd_file_get_name (finfo),
 						gnome_cmd_file_get_name (finfo2)) == 0)
 			{
-				gchar *cmd = g_strdup_printf (gnome_cmd_data_get_differ (),
-											  gnome_cmd_file_get_path (finfo),
-											  gnome_cmd_file_get_path (finfo2));
-
-				run_command (cmd, FALSE);
-				g_print (_("running \"%s\"\n"), cmd);
-				g_free (cmd);
+				found = TRUE;
+				break;
 			}
 			tmp = tmp->next;
 		}
+
+		p1 = g_shell_quote (gnome_cmd_file_get_real_path (finfo));
+
+		if (found)
+			p2 = g_shell_quote (gnome_cmd_file_get_real_path (finfo2));
+
+		cmd = g_strdup_printf (gnome_cmd_data_get_differ (), p1, p2?p2:"");
+
+		g_free (p1);
+		if (p2)
+			g_free (p2);
+
+		run_command (cmd, FALSE);
+		g_print (_("running \"%s\"\n"), cmd);
+		g_free (cmd);
 	}
 }
 
 
-void
-file_umount                        (GtkMenuItem     *menuitem,
-                                     gpointer        not_used)
-{
-	GnomeCmdConnection *c1, *c2, *home;
-	GnomeCmdFileSelector *active, *inactive;
-
-	active = gnome_cmd_main_win_get_active_fs (main_win);
-	inactive = gnome_cmd_main_win_get_inactive_fs (main_win);
-	
-	c1 = gnome_cmd_file_selector_get_connection (active);
-	c2 = gnome_cmd_file_selector_get_connection (inactive);
-	home = gnome_cmd_connection_get_home ();
-	
-	gnome_cmd_file_selector_set_connection (active, home, NULL);
-	if (c1 == c2)
-		gnome_cmd_file_selector_set_connection (inactive, home, NULL);
-
-	if (c1->type == CON_TYPE_DEV)
-		gnome_cmd_connection_umount (c1);
-	else
-		gnome_cmd_connection_free (c1);
-}
-
-
 void
 file_exit                           (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {
-	gtk_main_quit ();
+	gtk_widget_destroy (GTK_WIDGET (main_win));
 }
 
 
@@ -282,17 +277,11 @@ void
 edit_search                         (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {
-	GnomeCmdFileSelector *fs = get_current_fs ();
-	GnomeCmdConnection *con = gnome_cmd_file_selector_get_connection (fs);
+	GnomeCmdFileSelector *fs = get_active_fs ();
 	GtkWidget *dialog =
-		gnome_cmd_search_dialog_new (
-			gnome_cmd_connection_get_baseuri (con),
+		gnome_cmd_search_dialog_new (			
 			gnome_cmd_file_selector_get_directory (fs));
-	
 	gtk_widget_ref (dialog);
-	gtk_object_set_data_full (
-		GTK_OBJECT (main_win), "gnome_cmd_search_dialog", dialog,
-				(GtkDestroyNotify) gtk_widget_unref);
 	gtk_widget_show (dialog);
 }
 
@@ -301,7 +290,15 @@ void
 edit_quick_search                   (GtkMenuItem     *menuitem,
  								     gpointer        not_used)
 {
-	gnome_cmd_file_list_show_quicksearch (get_current_fl (), 0);
+	gnome_cmd_file_list_show_quicksearch (get_active_fl (), 0);
+}
+
+
+void
+edit_filter                         (GtkMenuItem     *menuitem,
+ 								     gpointer        not_used)
+{
+	gnome_cmd_file_selector_show_filter (get_active_fs (), 0);
 }
 
 
@@ -310,7 +307,7 @@ void
 mark_toggle                         (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {
-	gnome_cmd_file_list_toggle (get_current_fl ());
+	gnome_cmd_file_list_toggle (get_active_fl ());
 }
 
 
@@ -318,7 +315,7 @@ void
 mark_toggle_and_step                (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {
-	gnome_cmd_file_list_toggle_and_step (get_current_fl ());	
+	gnome_cmd_file_list_toggle_and_step (get_active_fl ());	
 }
 
 
@@ -326,7 +323,7 @@ void
 mark_select_all                     (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {
-	gnome_cmd_file_list_select_all (get_current_fl ());
+	gnome_cmd_file_list_select_all (get_active_fl ());
 }
 
 
@@ -334,7 +331,7 @@ void
 mark_unselect_all                   (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {
-	gnome_cmd_file_list_unselect_all (get_current_fl ());
+	gnome_cmd_file_list_unselect_all (get_active_fl ());
 }
 
 
@@ -342,7 +339,7 @@ void
 mark_select_with_pattern            (GtkMenuItem    *menuitem,
                                      gpointer        not_used)
 {
-	gnome_cmd_file_list_show_selpat_dialog (get_current_fl (), TRUE);
+	gnome_cmd_file_list_show_selpat_dialog (get_active_fl (), TRUE);
 }
 
 
@@ -350,7 +347,7 @@ void
 mark_unselect_with_pattern            (GtkMenuItem    *menuitem,
                                      gpointer        not_used)
 {
-	gnome_cmd_file_list_show_selpat_dialog (get_current_fl (), FALSE);
+	gnome_cmd_file_list_show_selpat_dialog (get_active_fl (), FALSE);
 }
 
 
@@ -358,7 +355,7 @@ void
 mark_invert_selection                 (GtkMenuItem    *menuitem,
                                      gpointer        not_used)
 {
-	gnome_cmd_file_list_invert_selection (get_current_fl ());
+	gnome_cmd_file_list_invert_selection (get_active_fl ());
 }
 
 
@@ -366,7 +363,7 @@ void
 mark_select_all_with_same_extension   (GtkMenuItem    *menuitem,
                                      gpointer        not_used)
 {
-	gnome_cmd_file_list_select_all_with_same_extension (get_current_fl ());
+	gnome_cmd_file_list_select_all_with_same_extension (get_active_fl ());
 }
 
 
@@ -374,7 +371,7 @@ void
 mark_unselect_all_with_same_extension   (GtkMenuItem    *menuitem,
                                      gpointer        not_used)
 {
-	gnome_cmd_file_list_unselect_all_with_same_extension (get_current_fl ());
+	gnome_cmd_file_list_unselect_all_with_same_extension (get_active_fl ());
 }
 
 
@@ -382,20 +379,38 @@ void
 mark_restore_selection                 (GtkMenuItem    *menuitem,
                                      gpointer        not_used)
 {
-	gnome_cmd_file_list_restore_selection (get_current_fl ());
+	gnome_cmd_file_list_restore_selection (get_active_fl ());
+}
+
+
+void
+mark_compare_directories               (GtkMenuItem     *menuitem,
+                                     gpointer        not_used)
+{
+	gnome_cmd_file_list_compare_directories ();
 }
 
 
 /************** View Menu **************/
 
+void
+view_conbuttons                        (GtkCheckMenuItem     *menuitem,
+ 							            gpointer              not_used)
+{
+	if (!GTK_WIDGET_REALIZED (main_win)) return;
+	gnome_cmd_data_set_conbuttons_visibility (menuitem->active);
+	gnome_cmd_file_selector_update_conbuttons_visibility (get_active_fs ());
+	gnome_cmd_file_selector_update_conbuttons_visibility (get_inactive_fs ());
+}
+
+
 void
 view_toolbar                           (GtkCheckMenuItem     *menuitem,
  							            gpointer              not_used)
 {
-	if (!GTK_WIDGET_REALIZED (main_win))
-		return;
-	gnome_cmd_main_win_set_toolbar_visibility (
-		main_win, menuitem->active);
+	if (!GTK_WIDGET_REALIZED (main_win)) return;
+	gnome_cmd_data_set_toolbar_visibility (menuitem->active);
+	gnome_cmd_main_win_update_toolbar_visibility (main_win);
 }
 
 
@@ -403,10 +418,41 @@ void
 view_buttonbar                         (GtkCheckMenuItem     *menuitem,
  							            gpointer              not_used)
 {
-	if (!GTK_WIDGET_REALIZED (main_win))
-		return;
-	gnome_cmd_main_win_set_buttonbar_visibility (
-		main_win, menuitem->active);
+	if (!GTK_WIDGET_REALIZED (main_win)) return;
+	gnome_cmd_data_set_buttonbar_visibility (menuitem->active);
+	gnome_cmd_main_win_update_buttonbar_visibility (main_win);
+}
+
+
+void
+view_cmdline                           (GtkCheckMenuItem     *menuitem,
+ 							            gpointer              not_used)
+{
+	if (!GTK_WIDGET_REALIZED (main_win)) return;
+	gnome_cmd_data_set_cmdline_visibility (menuitem->active);
+	gnome_cmd_main_win_update_cmdline_visibility (main_win);
+}
+
+
+void
+view_hidden_files                      (GtkCheckMenuItem     *menuitem,
+ 							            gpointer              not_used)
+{
+	if (!GTK_WIDGET_REALIZED (main_win)) return;
+	gnome_cmd_data_get_filter_settings()->hidden = !menuitem->active;
+	gnome_cmd_file_selector_reload (get_active_fs ());
+	gnome_cmd_file_selector_reload (get_inactive_fs ());
+}
+
+
+void
+view_backup_files                      (GtkCheckMenuItem     *menuitem,
+ 							            gpointer              not_used)
+{
+	if (!GTK_WIDGET_REALIZED (main_win)) return;
+	gnome_cmd_data_get_filter_settings()->backup = !menuitem->active;
+	gnome_cmd_file_selector_reload (get_active_fs ());
+	gnome_cmd_file_selector_reload (get_inactive_fs ());
 }
 
 
@@ -414,7 +460,7 @@ void
 view_back                              (GtkMenuItem     *menuitem,
 										gpointer        not_used)
 {
-	gnome_cmd_file_selector_back (get_current_fs ());
+	gnome_cmd_file_selector_back (get_active_fs ());
 }
 
 
@@ -422,7 +468,7 @@ void
 view_forward                           (GtkMenuItem     *menuitem,
 										gpointer        not_used)
 {
-	gnome_cmd_file_selector_forward (get_current_fs ());
+	gnome_cmd_file_selector_forward (get_active_fs ());
 }
 
 
@@ -430,7 +476,7 @@ void
 view_refresh                           (GtkMenuItem     *menuitem,
 										gpointer        not_used)
 {
-	gnome_cmd_file_selector_reload (get_current_fs ());
+	gnome_cmd_file_selector_reload (get_active_fs ());
 }
 
 
@@ -450,7 +496,7 @@ void
 options_edit_mime_types             (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {
-	run_command ("gnomecc --capplet=file-types-capplet", FALSE);
+	edit_mimetypes (NULL, FALSE);
 }
 
 
@@ -463,22 +509,19 @@ options_save_position               (GtkMenuItem     *menuitem,
 
 
 
-/************** FTP Menu **************/
+/************** Connections Menu **************/
 void
-ftp_connect                         (GtkMenuItem     *menuitem,
+connections_ftp_connect             (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {
 	GtkWidget *dialog = gnome_cmd_ftp_dialog_new ();
 	gtk_widget_ref (dialog);
-	gtk_object_set_data_full (
-		GTK_OBJECT (main_win), "gnome_cmd_ftp_dialog", dialog,
-		(GtkDestroyNotify) gtk_widget_unref);
 	gtk_widget_show (dialog);
 }
 
 
 void
-ftp_quick_connect                   (GtkMenuItem     *menuitem,
+connections_ftp_quick_connect       (GtkMenuItem     *menuitem,
                                      gpointer        not_used)
 {
 	show_ftp_quick_connect_dialog ();
@@ -486,30 +529,45 @@ ftp_quick_connect                   (GtkMenuItem     *menuitem,
 
 
 void
-ftp_disconnect                      (GtkMenuItem     *menuitem,
-                                     gpointer        not_used)
+connections_change                  (GtkMenuItem *menuitem,
+                                     GnomeCmdCon *con)
 {
-	GnomeCmdConnection *con =
-		gnome_cmd_file_selector_get_connection (get_current_fs ());
-
-	if (con->type == CON_TYPE_FTP) {
-		GnomeCmdFileSelector *fs;
-		
-		fs = gnome_cmd_main_win_get_right_fs (main_win);
-		if (gnome_cmd_file_selector_get_connection (fs) == con)
-			gnome_cmd_file_selector_set_connection (
-				fs, gnome_cmd_connection_get_home (), NULL);
-
-		fs = gnome_cmd_main_win_get_left_fs (main_win);
-		if (gnome_cmd_file_selector_get_connection (fs) == con)
-			gnome_cmd_file_selector_set_connection (
-				fs, gnome_cmd_connection_get_home (), NULL);
-		
-		gnome_cmd_connection_free (con);
-	}
+	gnome_cmd_file_selector_set_connection (get_active_fs (), con, NULL);
 }
 
 
+void
+connections_close           (GtkMenuItem     *menuitem,
+                             GnomeCmdCon *con)
+{
+	GnomeCmdCon *c1, *c2, *home;
+	GnomeCmdFileSelector *active, *inactive;
+
+	active = gnome_cmd_main_win_get_active_fs (main_win);
+	inactive = gnome_cmd_main_win_get_inactive_fs (main_win);
+	
+	c1 = gnome_cmd_file_selector_get_connection (active);
+	c2 = gnome_cmd_file_selector_get_connection (inactive);
+	home = gnome_cmd_con_list_get_home (gnome_cmd_data_get_con_list ());
+	
+	if (con == c1)
+		gnome_cmd_file_selector_set_connection (active, home, NULL);
+	if (con == c2)
+		gnome_cmd_file_selector_set_connection (inactive, home, NULL);
+	
+	gnome_cmd_con_close (con);
+}
+
+
+void
+connections_close_current           (GtkMenuItem     *menuitem,
+                                     gpointer         not_used)
+{
+	GnomeCmdCon *con = gnome_cmd_file_selector_get_connection (get_active_fs ());
+
+	connections_close (menuitem, con);
+}
+
 
 /************** Bookmarks Menu **************/
 
@@ -527,12 +585,22 @@ bookmarks_edit                      (GtkMenuItem     *menuitem,
 {
 	GtkWidget *dialog = gnome_cmd_bookmark_dialog_new ();
 	gtk_widget_ref (dialog);
-	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (main_win));
 	gtk_widget_show (dialog);
 }
 
 
 
+/************** Plugins Menu **************/
+
+void
+plugins_configure                   (GtkMenuItem     *menuitem,
+                                     gpointer        not_used)
+{
+	plugin_manager_show ();
+}
+
+
+
 /************** Help Menu **************/
 
 void
@@ -541,7 +609,6 @@ help_help                           (GtkMenuItem     *menuitem,
 {
 	gchar *url = g_strdup_printf("file:%s/C/index.html", HELP_DIR);
 
-	gnome_help_goto (NULL, url);	
 	g_free(url);	
 }
 
@@ -552,23 +619,29 @@ help_about                          (GtkMenuItem     *menuitem,
 {
 	gchar *authors[] = {
 		"Marcus Bjurman <marbj499@student.liu.se>",
-		"Piotr Eljasiak <epiotr@idea.net.pl>",
+		"Piotr Eljasiak <epiotr@use.pl>",
 		NULL
 	};
 
+	gchar *docers[] = {
+		"Marcus Bjurman <marbj499@student.liu.se>",
+		"Piotr Eljasiak <epiotr@use.pl>",
+		NULL
+	};
+	
 	GtkWidget *dialog = gnome_about_new (
 		"Gnome Commander",
 		VERSION,
-		"Copyright: All contributors 2001-2003",
+		"Copyright (C) 2001-2003 Marcus Bjurman",
+		_("A fast and powerfull file manager for the GNOME desktop"),
 		(const gchar**)authors,
-		"This is a GPL licensed program",
+		(const gchar**)docers,
+		/* xgettext:
+		 * Replace this with your name when translating  */
+		_("No translation is currently in use."), 
 		NULL);
 
 	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
 	gtk_widget_ref (dialog);
-	gtk_object_set_data_full (
-		GTK_OBJECT (main_win), "about_dialog", dialog,
-		(GtkDestroyNotify) gtk_widget_unref);
-
 	gtk_widget_show (dialog);
 }
