@@ -20,16 +20,24 @@
 #include <config.h>
 #include <locale.h>
 #include <errno.h>
+#include <dirent.h>
 #include "gnome-cmd-includes.h"
 #include "utils.h"
 #include "gnome-cmd-data.h"
+#include "gnome-cmd-plain-path.h"
 #include "imageloader.h"
-#include "gnome-cmd-file-funcs.h"
-#include "gnome-cmd-main-win-types.h"
+#include "gnome-cmd-file.h"
+#include "gnome-cmd-main-win.h"
 #include "gnome-cmd-xfer.h"
-#include "gnome-cmd-dir-funcs.h"
 
 
+typedef struct
+{
+	GnomeCmdFile *finfo;
+	GtkWidget *dialog;
+	gpointer *args;
+	
+} TmpDlData;
 #ifdef HAVE_LOCALE_H
 extern struct lconv *locale_information;
 #endif
@@ -41,8 +49,7 @@ extern gchar *debug_flags;
 static gchar *tmp_file_dir = NULL;
 
 
-static gboolean
-debug_enabled (char flag)
+gboolean DEBUG_ENABLED (gchar flag)
 {
     if (debug_flags != NULL)
        return (strchr(debug_flags, flag) != 0);
@@ -60,16 +67,18 @@ debug_enabled (char flag)
  * i: imageloader
  * p: directory pool
  * l: directory listings
- * m: mounting
- * n: FAM debug
+ * m: connection debug.
+ * n: directory-monitoring debug
+ * s: smb network browser debugging.
  * w: widget_lookup debugging
  * y: brief mime-based imageload debugging
  * z: detailed mime-based imageload debugging
+ * x: xfer
  * 
  */
-void DEBUG (char flag, const char *format, ...)
+void DEBUG (gchar flag, const gchar *format, ...)
 {
-    if (debug_enabled (flag)) {
+    if (DEBUG_ENABLED (flag)) {
 		va_list ap;		
 		va_start(ap, format);
 		fprintf (stderr, "[%c%c] ", flag-32, flag-32);
@@ -103,10 +112,13 @@ void run_command_indir (const gchar *in_command, const gchar *dir,
 	gchar *command;
 	
 	if (term) {
-		gchar *tmp;
+		gchar *tmp, *arg;
 
-		tmp = g_strdup_printf ("sh -c '%s; echo ; echo --------------------------------------------------------------; echo The program has now finished, press Enter to close this window; read input' &", in_command);
-		command = g_strdup_printf (gnome_cmd_data_get_term (), tmp);
+		tmp = g_strdup_printf ("%s; %s/bin/gcmd-block", in_command, PREFIX);
+		arg = g_shell_quote (tmp);
+		command = g_strdup_printf (gnome_cmd_data_get_term (), arg);
+		g_free (arg);
+		g_free (tmp);
 	}
 	else
 		command = g_strdup (in_command);
@@ -117,38 +129,110 @@ void run_command_indir (const gchar *in_command, const gchar *dir,
 }
 
 
-void create_error_dialog (const gchar *msg, ...)
+const char **
+convert_varargs_to_name_array (va_list args)
 {
-    va_list     argptr;
-    gchar        string[1024];
+	GPtrArray *resizeable_array;
+	const char *name;
+	const char **plain_ole_array;
+	
+	resizeable_array = g_ptr_array_new ();
 
-    va_start (argptr, msg);
-    vsprintf (string, msg, argptr);
-    va_end (argptr);
+	do {
+		name = va_arg (args, const char *);
+		g_ptr_array_add (resizeable_array, (gpointer) name);
+	} while (name != NULL);
 
-    gnome_error_dialog (string);
+	plain_ole_array = (const char **) resizeable_array->pdata;
+	
+	g_ptr_array_free (resizeable_array, FALSE);
+
+	return plain_ole_array;
 }
 
 
-void create_warning_dialog (const gchar *msg, ...)
+static gboolean
+delete_event_callback (gpointer data,
+		       gpointer user_data)
 {
-    va_list     argptr;
-    gchar        string[1024];
-
-    va_start (argptr, msg);
-    vsprintf (string, msg, argptr);
-    va_end (argptr);
-
-    gnome_warning_dialog (string);
+	g_return_val_if_fail (GTK_IS_DIALOG (data), FALSE);
+	
+	gtk_signal_emit_stop_by_name (GTK_OBJECT (data), "delete_event");
+	
+	return TRUE;
 }
 
 
-void create_question_dialog (const gchar *msg, gpointer callback)
+static gboolean
+on_run_dialog_keypress (GtkWidget *dialog, GdkEventKey *event, gpointer data)
 {
-    gnome_question_dialog_modal (msg, (GnomeReplyCallback)callback, NULL);
+	if (event->keyval == GDK_Escape) {
+		gtk_dialog_response (GTK_DIALOG (dialog), GTK_RESPONSE_NONE);
+		return TRUE;
+	}
+
+	return FALSE;
 }
 
 
+gint
+run_simple_dialog (GtkWidget *parent, gboolean ignore_close_box,
+				   GtkMessageType msg_type,
+				   const char *text, const char *title, ...)
+{
+	va_list button_title_args;
+	const char **button_titles;
+	GtkWidget *dialog;
+//	GtkWidget *top_widget;
+	int result, i;
+
+	
+	/* Create the dialog. */
+	va_start (button_title_args, title);
+	button_titles = convert_varargs_to_name_array (button_title_args);
+	va_end (button_title_args);
+
+	dialog = gtk_message_dialog_new (
+		GTK_WINDOW (main_win), GTK_DIALOG_MODAL,
+		msg_type, GTK_BUTTONS_NONE, text);
+	if (title)
+		gtk_window_set_title (GTK_WINDOW (dialog), title);
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+
+	for ( i=0 ; button_titles[i] != NULL ; i++ )
+		gtk_dialog_add_button (GTK_DIALOG (dialog), button_titles[i], i);
+									 
+	g_free (button_titles);
+	
+	/* Allow close. */
+	if (ignore_close_box) {
+		gtk_signal_connect (GTK_OBJECT (dialog),
+							"delete_event",
+							GTK_SIGNAL_FUNC (delete_event_callback),
+							NULL);
+	}
+	else {
+		gtk_signal_connect (GTK_OBJECT (dialog),
+							"key-press-event",
+							GTK_SIGNAL_FUNC (on_run_dialog_keypress),
+							dialog);
+	}
+	
+	gtk_window_set_wmclass (GTK_WINDOW (dialog), "dialog", "Eel");
+
+	
+	/* Run it. */
+	do
+	{
+		gtk_widget_show (dialog);
+		result = gtk_dialog_run (GTK_DIALOG (dialog));
+	} while (ignore_close_box && result == GTK_RESPONSE_DELETE_EVENT);
+
+	gtk_widget_destroy(dialog);
+
+	return result;
+}
+
 
 gboolean string2int (const gchar *s, gint *i)
 {
@@ -216,22 +300,6 @@ char *int2string (int i)
 }
 
 
-
-gchar *concat_dir_file (const gchar *dir, const gchar *file)
-{
-	if (!(dir && file))
-		return NULL;
-
-	if (strcmp (dir, "/") == 0)
-		return g_strdup_printf ("/%s", file);
-
-	if (dir[strlen (dir) - 1] == '/')
-		return g_strdup_printf ("%s%s", dir, file);
-
-	return g_strdup_printf ("%s/%s", dir, file);
-}
-
-
 gchar *str_uri_basename (const gchar *uri)
 {
 	int i, len, last_slash=0;
@@ -357,7 +425,7 @@ const gchar *size2string (GnomeVFSFileSize size,
 	
     if (size_disp_mode == GNOME_CMD_SIZE_DISP_MODE_POWERED)
     {
-        gchar *prefixes[5] = {"B  ","kB ","MB ","GB ","TB "};
+        gchar *prefixes[5] = {"B ","kB ","MB ","GB ","TB "};
         gint i=0;
         gdouble dsize = (gdouble)size;
         
@@ -444,7 +512,8 @@ no_mime_app_found_error (gchar *mime_type)
 
 	msg = g_strdup_printf (_("No default application found for the mime-type %s.\nOpen the \"File types and programs\" page in the Control Center to add one."),
 						   mime_type);
-	gnome_error_dialog (msg);
+	create_error_dialog (msg);
+	g_free (msg);
 }
 
 
@@ -453,8 +522,11 @@ static void do_mime_exec_single (gpointer *args)
 	gchar *cmd;
 	GnomeCmdApp *app = (GnomeCmdApp*)args[0];
 	gchar *path = (gchar*)args[1];
+	gchar *arg;
 
-	cmd = g_strdup_printf ("%s \"%s\"", gnome_cmd_app_get_command (app), path);
+	arg = g_shell_quote (path);
+	cmd = g_strdup_printf ("%s %s", gnome_cmd_app_get_command (app), arg);
+	g_free (arg);
 	run_command (cmd, gnome_cmd_app_get_requires_terminal (app));
 	g_free (cmd);
 	g_free (path);
@@ -463,6 +535,44 @@ static void do_mime_exec_single (gpointer *args)
 }
 
 
+static void
+on_tmp_download_response (GtkWidget *w, gint id, TmpDlData *dldata)
+{
+	if (id == GTK_RESPONSE_YES) {
+		GnomeVFSURI *src_uri, *dest_uri;
+		GnomeCmdCon *con;
+		GnomeCmdPath *path;
+		gchar *path_str;
+
+		path_str = get_temp_download_filepath (gnome_cmd_file_get_name (dldata->finfo));
+		if (!path_str) return;
+		dldata->args[1] = (gpointer)path_str;
+
+		src_uri = gnome_vfs_uri_dup (gnome_cmd_file_get_uri (dldata->finfo));
+		
+		path = gnome_cmd_plain_path_new (path_str);
+		con = gnome_cmd_con_list_get_home (
+			gnome_cmd_data_get_con_list ());
+		dest_uri = gnome_cmd_con_create_uri (con, path);
+		gtk_object_destroy (GTK_OBJECT (path));
+			
+		gnome_cmd_xfer_tmp_download (src_uri,
+									 dest_uri,
+									 GNOME_VFS_XFER_FOLLOW_LINKS,
+									 GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE,
+									 GTK_SIGNAL_FUNC (do_mime_exec_single),
+									 dldata->args);
+	}
+	else {
+		gnome_cmd_app_free ((GnomeCmdApp*)dldata->args[0]);
+		g_free (dldata->args);
+	}
+
+	g_free (dldata);
+	gtk_widget_destroy (dldata->dialog);
+}
+
+
 void mime_exec_single (GnomeCmdFile *finfo)
 {
 	gpointer *args;
@@ -475,11 +585,54 @@ void mime_exec_single (GnomeCmdFile *finfo)
 	if (!finfo->info->mime_type)
 		return;
 
-	if (strcmp (finfo->info->mime_type, "application/x-executable-binary") == 0) {
-		if (gnome_vfs_uri_is_local (gnome_cmd_file_get_uri (finfo)))
-			run_command_indir (gnome_cmd_file_get_path (finfo), gnome_cmd_dir_get_path (finfo->dir),
-							   app_needs_terminal (finfo));
-		return;
+	/* Check if the file is a binary executable that lacks the executable bit
+	 */
+	if (!gnome_cmd_file_is_executable (finfo)) {
+		if (gnome_cmd_file_has_mime_type (finfo, "application/x-executable-binary")) {
+			gchar *fname = get_utf8 (finfo->info->name);
+			gchar *msg = g_strdup_printf (_("\"%s\" seems to be a binary executable file but it lacks the executable bit. Do you want to set it and then run the file?"), fname);
+			gint ret = run_simple_dialog (
+				GTK_WIDGET (main_win), FALSE, GTK_MESSAGE_QUESTION, msg,
+				_("Make Executable?"), 
+				_("Cancel"), _("OK"), NULL);
+			g_free (fname);
+			g_free (msg);
+
+			if (ret == 1) {
+				GnomeVFSResult res = gnome_cmd_file_chmod (
+					finfo, finfo->info->permissions|GNOME_VFS_PERM_USER_EXEC);
+				if (res != GNOME_VFS_OK)
+					return;
+			}
+			else
+				return;
+		}
+	}
+
+
+	/* If the file is executable but not a binary file, check if the user wants to
+	 * exec it or open it.
+	 */
+	if (gnome_cmd_file_is_executable (finfo)) {
+		if (gnome_cmd_file_has_mime_type (finfo, "application/x-executable-binary")) {
+			gnome_cmd_file_execute (finfo);
+			return;
+		}
+		else if (gnome_cmd_file_mime_begins_with (finfo, "text/")) {
+			gchar *fname = get_utf8 (finfo->info->name);
+			gchar *msg = g_strdup_printf (_("\"%s\" is an executable text file. Do you want to run it, or display it's contents?"), fname);
+			gint ret = run_simple_dialog (
+				GTK_WIDGET (main_win), FALSE, GTK_MESSAGE_QUESTION, msg, _("Run or Display"), 
+				_("Cancel"), _("Display"), _("Run"), NULL);
+			g_free (fname);
+			g_free (msg);
+
+			if (ret != 1) {
+				if (ret == 2)
+					gnome_cmd_file_execute (finfo);
+				return;
+			}
+		}
 	}
 
 	vfs_app = gnome_vfs_mime_get_default_application (finfo->info->mime_type);
@@ -493,9 +646,9 @@ void mime_exec_single (GnomeCmdFile *finfo)
 
 	args = g_new (gpointer, 2);
 
-	if (gnome_vfs_uri_is_local (gnome_cmd_file_get_uri (finfo))) {
+	if (gnome_cmd_file_is_local (finfo)) {
 		args[0] = (gpointer)app;
-		args[1] = (gpointer)g_strdup (gnome_cmd_file_get_path (finfo));
+		args[1] = (gpointer)g_strdup (gnome_cmd_file_get_real_path (finfo));
 		do_mime_exec_single (args);
 	}
 	else {
@@ -506,26 +659,21 @@ void mime_exec_single (GnomeCmdFile *finfo)
 			do_mime_exec_single (args);
 		}
 		else {
-			gchar *dest_uri_path;
-			GnomeVFSURI *src_uri, *dest_uri;
-			gchar *path = get_temp_download_filepath (gnome_cmd_file_get_name (finfo));
-			
+			gchar *msg = g_strdup_printf (_("%s does not know how to open remote files. Do you want to download the file to a temporary location and then open it?"), gnome_cmd_app_get_name (app));
+			GtkWidget *dialog = gtk_message_dialog_new (
+				GTK_WINDOW (main_win), GTK_DIALOG_MODAL,
+				GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, msg);
+			TmpDlData *dldata = g_new (TmpDlData, 1);
 			args[0] = (gpointer)app;
-			args[1] = (gpointer)path;
+			dldata->finfo = finfo;
+			dldata->dialog = dialog;
+			dldata->args = args;
+			gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 
-			src_uri = gnome_vfs_uri_dup (gnome_cmd_file_get_uri (finfo));
-			dest_uri_path = g_strdup_printf ("file://%s", path);
-			dest_uri = gnome_vfs_uri_new (dest_uri_path);
-			g_free (dest_uri_path);
-			
-			g_printerr ("Copying to: %s\n", path);
-	
-			gnome_cmd_xfer_tmp_download (src_uri,
-										 dest_uri,
-										 GNOME_VFS_XFER_FOLLOW_LINKS,
-										 GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE,
-										 do_mime_exec_single,
-										 args);
+			gtk_signal_connect (GTK_OBJECT (dialog), "response",
+								GTK_SIGNAL_FUNC (on_tmp_download_response), dldata);
+			gtk_widget_show (dialog);
+			g_free (msg);
 		}
 	}
 }
@@ -537,20 +685,24 @@ static void do_mime_exec_multiple (gpointer *args)
 	GnomeCmdApp *app = (GnomeCmdApp*)args[0];
 	GList *files = (GList*)args[1];
 
-	cmd = g_strdup_printf ("%s ", gnome_cmd_app_get_command (app));
+	if (files) {
+		cmd = g_strdup_printf ("%s ", gnome_cmd_app_get_command (app));
 
-	while (files) {
-		gchar *path = (gchar*)files->data;
-		gchar *tmp = cmd;
-		cmd = g_strdup_printf ("%s \"%s\" ", tmp, path);
-		g_free (path);
-		g_free (tmp);
-		files = files->next;
+		while (files) {
+			gchar *path = (gchar*)files->data;
+			gchar *tmp = cmd;
+			gchar *arg = g_shell_quote (path);
+			cmd = g_strdup_printf ("%s %s", tmp, arg);
+			g_free (arg);
+			g_free (path);
+			g_free (tmp);
+			files = files->next;
+		}
+
+		run_command (cmd, gnome_cmd_app_get_requires_terminal (app));
+		g_free (cmd);
+		g_list_free (files);
 	}
-
-	run_command (cmd, gnome_cmd_app_get_requires_terminal (app));
-	g_free (cmd);
-	g_list_free (files);
 	gnome_cmd_app_free (app);
 	g_free (args);
 }
@@ -562,6 +714,8 @@ void mime_exec_multiple (GList *files, GnomeCmdApp *app)
 	GList *dest_uri_list = NULL;
 	gpointer *args;
 	GList *local_files = NULL;
+	gboolean asked = FALSE;
+	gint retid;
 	
 	g_return_if_fail (files != NULL);
 	g_return_if_fail (app != NULL);
@@ -570,25 +724,42 @@ void mime_exec_multiple (GList *files, GnomeCmdApp *app)
 		GnomeCmdFile *finfo = (GnomeCmdFile*)files->data;
 
 		if (gnome_vfs_uri_is_local (gnome_cmd_file_get_uri (finfo)))
-			local_files = g_list_append (local_files, g_strdup (gnome_cmd_file_get_path (finfo)));
+			local_files = g_list_append (local_files, g_strdup (gnome_cmd_file_get_real_path (finfo)));
 		else {
 			if (gnome_cmd_app_get_handles_uris (app)
 				&& gnome_cmd_data_get_honor_expect_uris()) {
 				local_files = g_list_append (local_files,  g_strdup (gnome_cmd_file_get_uri_str (finfo)));
 			}
 			else {
-				gchar *dest_uri_path;
-				GnomeVFSURI *src_uri, *dest_uri;
-				gchar *path = get_temp_download_filepath (gnome_cmd_file_get_name (finfo));
-			
-				src_uri = gnome_vfs_uri_dup (gnome_cmd_file_get_uri (finfo));
-				dest_uri_path = g_strdup_printf ("file://%s", path);
-				dest_uri = gnome_vfs_uri_new (dest_uri_path);
-				g_free (dest_uri_path);
+				if (!asked) {
+					gchar *msg = g_strdup_printf (
+						_("%s does not know how to open remote files. Do you want to download the files to a temporary location and then open them?"), gnome_cmd_app_get_name (app));
+					retid = run_simple_dialog (
+						GTK_WIDGET (main_win), TRUE, GTK_MESSAGE_QUESTION,
+						msg, "", _("No"), _("Yes"), NULL);
+					asked = TRUE;
+				}
 
-				src_uri_list = g_list_append (src_uri_list, src_uri);
-				dest_uri_list = g_list_append (dest_uri_list, dest_uri);
-				local_files = g_list_append (local_files, path);
+				if (retid == 1) {
+					GnomeCmdCon *con;
+					GnomeCmdPath *path;
+					GnomeVFSURI *src_uri, *dest_uri;
+					gchar *path_str;
+
+					path_str = get_temp_download_filepath (gnome_cmd_file_get_name (finfo));
+					if (!path_str) return;
+			
+					src_uri = gnome_vfs_uri_dup (gnome_cmd_file_get_uri (finfo));
+					path = gnome_cmd_plain_path_new (path_str);
+					con = gnome_cmd_con_list_get_home (
+						gnome_cmd_data_get_con_list ());
+					dest_uri = gnome_cmd_con_create_uri (con, path);
+					gtk_object_destroy (GTK_OBJECT (path));
+
+					src_uri_list = g_list_append (src_uri_list, src_uri);
+					dest_uri_list = g_list_append (dest_uri_list, dest_uri);
+					local_files = g_list_append (local_files, path_str);
+				}
 			}
 		}
 
@@ -607,7 +778,7 @@ void mime_exec_multiple (GList *files, GnomeCmdApp *app)
 			dest_uri_list,
 			GNOME_VFS_XFER_FOLLOW_LINKS,
 			GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE,
-			do_mime_exec_multiple,
+			GTK_SIGNAL_FUNC (do_mime_exec_multiple),
 			args);
 	}
 	else {
@@ -686,14 +857,17 @@ strings_to_uris (gchar *data)
 	
 	for ( i=0 ; filenames[i] != NULL ; i++ ) {
 		gchar *fn;
-		
+		GnomeVFSURI *uri;
+
 		if (i == STRINGS_TO_URIS_CHUNK) {
 			uri_list = g_list_concat (uri_list, strings_to_uris (filenames[i]));
 			break;
 		}
 
 		fn = g_strdup (filenames[i]);
-		uri_list = g_list_append (uri_list, gnome_vfs_uri_new (fn));
+		uri = gnome_vfs_uri_new (fn);
+		if (uri)
+			uri_list = g_list_append (uri_list, uri);
 		g_free (fn);
 	}
 
@@ -719,8 +893,7 @@ GnomeVFSFileSize calc_tree_size (const GnomeVFSURI *dir_uri)
 	result = gnome_vfs_directory_list_load (
 		&list,
 		dir_uri_str,
-		infoOpts,
-		NULL);
+		infoOpts);
 
 	if (result != GNOME_VFS_OK)
 		return 0;
@@ -814,7 +987,8 @@ create_nice_size_str (GnomeVFSFileSize size)
 
 gchar* quote_if_needed (const gchar *in)
 {
-	return strpbrk(in," ;&$'\"?")==NULL ? g_strdup (in) : g_strdup_printf ("'%s'", in);
+	//return strpbrk(in," ;&$'\"?")==NULL ? g_strdup (in) : g_strdup_printf ("'%s'", in);
+	return g_shell_quote (in);
 }
 
 
@@ -931,11 +1105,14 @@ GList *app_get_linked_libs (GnomeCmdFile *finfo)
 	FILE *fd;
 	gchar *cmd, *s;
 	gchar tmp[256];
+	gchar *arg;
 	GList *libs = NULL;
 
 	g_return_val_if_fail (GNOME_CMD_IS_FILE (finfo), NULL);
-	
-	cmd = g_strdup_printf ("ldd %s", gnome_cmd_file_get_path (finfo));
+
+	arg = g_shell_quote (gnome_cmd_file_get_real_path (finfo));
+	cmd = g_strdup_printf ("ldd %s", arg);
+	g_free (arg);
 	fd = popen (cmd, "r");
 	g_free (cmd);
 
@@ -959,6 +1136,9 @@ gboolean app_needs_terminal (GnomeCmdFile *finfo)
 	GList *tmp, *libs;
 	gboolean need_term = TRUE;
 
+	if (strcmp (finfo->info->mime_type, "application/x-executable-binary"))
+		return TRUE;
+
 	libs = tmp = app_get_linked_libs (finfo);
 	if  (!libs) return FALSE;
 
@@ -981,9 +1161,11 @@ gboolean app_needs_terminal (GnomeCmdFile *finfo)
 
 gchar *get_temp_download_filepath (const gchar *fname)
 {
+	const gchar *tmp_dir = g_get_tmp_dir ();
+	
 	if (!tmp_file_dir) {
 		gchar *tmp_file_dir_template = g_strdup_printf ("gcmd-%s-XXXXXX", g_get_user_name());
-		chdir ("/tmp");
+		chdir (tmp_dir);
 		tmp_file_dir = mkdtemp (tmp_file_dir_template);
 		if (!tmp_file_dir) {
 			g_free (tmp_file_dir_template);
@@ -995,14 +1177,18 @@ gchar *get_temp_download_filepath (const gchar *fname)
 		}
 	}
 
-	return g_strdup_printf ("/tmp/%s/%s", tmp_file_dir, fname);
+	return g_build_path ("/", tmp_dir, tmp_file_dir, fname, NULL);
 }
 
 
 void remove_temp_download_dir (void)
 {
+	const gchar *tmp_dir = g_get_tmp_dir ();
+	
 	if (tmp_file_dir) {
-		gchar *command = g_strdup_printf ("rm -rf /tmp/%s", tmp_file_dir);
+		gchar *path = g_build_path ("/", tmp_dir, tmp_file_dir, NULL);
+		gchar *command = g_strdup_printf ("rm -rf %s", path);
+		g_free (path);
 		system (command);
 		g_free (command);
 	}
@@ -1012,7 +1198,8 @@ void remove_temp_download_dir (void)
 GtkWidget *
 create_ui_pixmap (GtkWidget *window,
 				  GnomeUIPixmapType pixmap_type, 
-				  gconstpointer pixmap_info)
+				  gconstpointer pixmap_info,
+				  GtkIconSize size)
 {
 	GtkWidget *pixmap;
 	char *name;
@@ -1021,12 +1208,12 @@ create_ui_pixmap (GtkWidget *window,
 
 	switch (pixmap_type) {
 	case GNOME_APP_PIXMAP_STOCK:
-		pixmap = gnome_stock_pixmap_widget (window, pixmap_info);
+		pixmap = gtk_image_new_from_stock (pixmap_info, size);
 		break;
 
 	case GNOME_APP_PIXMAP_DATA:
 		if (pixmap_info)
-			pixmap = gnome_pixmap_new_from_xpm_d ((char**)pixmap_info);
+			pixmap = gnome_pixmap_new_from_xpm_d ((const char**)pixmap_info);
 
 		break;
 
@@ -1055,3 +1242,146 @@ create_ui_pixmap (GtkWidget *window,
 }
 
 
+static void
+transform (gchar *s, gchar from, gchar to)
+{
+	gint i;
+	gint len = strlen (s);
+	
+	for ( i=0 ; i<len ; i++ )
+		if (s[i] == from) s[i] = to;
+}
+
+
+gchar *unix_to_unc (const gchar *path)
+{
+	gchar *out;
+	
+	g_return_val_if_fail (path != NULL, NULL);
+	g_return_val_if_fail (path[0] == '/', NULL);
+
+	out = malloc (strlen(path)+2);
+	out[0] = '\\';
+	strcpy (out+1, path);
+	transform (out+1, '/', '\\');
+
+	return out;
+}
+
+gchar *unc_to_unix (const gchar *path)
+{
+	gchar *out;
+	
+	g_return_val_if_fail (path != NULL, NULL);
+	g_return_val_if_fail (path[0] == '\\', NULL);
+	g_return_val_if_fail (path[1] == '\\', NULL);
+
+	out = malloc (strlen(path));
+	strcpy (out, path+1);
+	transform (out, '\\', '/');
+
+	return out;
+}
+
+
+GdkColor *gdk_color_new (gushort r, gushort g, gushort b)
+{
+	GdkColor *c = g_new (GdkColor, 1);
+	c->pixel = 0;
+	c->red = r;
+	c->green = g;
+	c->blue = b;
+	return c;
+}
+
+
+GList *file_list_to_uri_list (GList *files)
+{
+	GList *uris = NULL;
+
+	while (files) {
+		GnomeCmdFile *finfo = GNOME_CMD_FILE (files->data);
+		GnomeVFSURI *uri = gnome_cmd_file_get_uri (finfo);
+
+		if (!uri)
+			g_warning ("NULL uri!!!\n");
+		else {
+			uris = g_list_append (uris, uri);
+		}
+		
+		files = files->next;
+	}
+
+	return uris;
+}
+
+
+GList *file_list_to_info_list (GList *files)
+{
+	GList *infos = NULL;
+
+	if (!files) return NULL;
+
+	while (files) {
+		GnomeCmdFile *finfo = GNOME_CMD_FILE (files->data);
+		infos = g_list_append (infos, finfo->info);
+		files = files->next;
+	}
+
+	return infos;
+}
+
+
+gboolean
+create_dir_if_needed (const gchar *dpath)
+{
+	DIR *dir;
+	
+	g_return_val_if_fail (dpath, FALSE);
+	
+	if ((dir = opendir (dpath)) == NULL) {
+		if (errno == ENOENT) {
+			g_print (_("Creating directory %s... "), dpath);
+			if (mkdir (dpath, S_IRUSR|S_IWUSR|S_IXUSR) == 0) {
+				return TRUE;
+			}
+			else {
+				gchar *msg = g_strdup_printf (_("Failed to create the directory %s"), dpath);
+				perror (msg);
+				g_free (msg);
+			}
+		}
+		else {
+			gchar *msg = g_strdup_printf (_("Couldn't read from the directory %s: %s\n"),
+										  dpath, strerror (errno));
+			warn_print (msg);
+			g_free (msg);
+		}
+		
+		return FALSE;
+	}
+
+	closedir (dir);
+	return TRUE;
+}
+
+
+void
+edit_mimetypes (const gchar *mime_type, gboolean blocking)
+{
+	gchar *cmd;
+	gchar *arg;
+
+	if (!mime_type)
+		mime_type = "";
+
+	arg = g_shell_quote (mime_type);
+	cmd = g_strdup_printf (
+		"gnome-file-types-properties %s\n", arg);
+	g_free (arg);
+	if (blocking)
+		system (cmd);
+	else
+		run_command (cmd, FALSE);
+	g_free (cmd);
+}
